import os
import json
import logging
import logging.config
import logging.handlers
import sys

from enum import Enum


def config_logger(loader_path,
                  log_name='component',
                  configure_root=False,
                  root_level=logging.WARN,
                  max_files=5,
                  max_size=5242880,
                  add_error_handler=False):
    real_path = os.path.realpath(loader_path)
    if os.path.isdir(real_path):
        dir_path = real_path
    else:
        dir_path = os.path.dirname(real_path)

    filename = os.path.join(dir_path, log_name + '.log')
    filename_error = os.path.join(dir_path, log_name + '.error.log')

    formatter = logging.Formatter("%(asctime)-6s: %(name)s - %(levelname)s - "
                                  "%(thread)d - %(threadName)s - %(message)s")

    comp_logger = logging.getLogger(log_name)
    comp_logger.setLevel(logging.DEBUG)

    rotating_file_logger = logging.handlers.RotatingFileHandler(filename, 'a', max_size, max_files, encoding="utf-8")
    rotating_file_logger.setLevel(logging.DEBUG)
    rotating_file_logger.setFormatter(formatter)
    comp_logger.addHandler(rotating_file_logger)

    if add_error_handler:
        rotating_file_logger_error = logging.handlers.RotatingFileHandler(filename_error,
                                                                          'a',
                                                                          max_size,
                                                                          max_files,
                                                                          encoding="utf-8")
        rotating_file_logger_error.setLevel(logging.ERROR)
        rotating_file_logger_error.setFormatter(formatter)
        comp_logger.addHandler(rotating_file_logger_error)

    if configure_root:
        root_logger = logging.getLogger()
        root_logger.addHandler(rotating_file_logger)
        root_logger.setLevel(root_level)


def import_pydevd(loader_path, port, suspend=False):
    # type: (str, int, bool) -> None
    if "DEBUG" in os.environ:
        dir_path = os.path.dirname(os.path.realpath(loader_path))
        filename = os.path.join(dir_path, "debug.txt")
        if os.path.isfile(filename):
            with open(filename, "r") as debug_file:
                pydev_path = debug_file.read()
            if os.path.exists(pydev_path):
                try:
                    sys.path.index(pydev_path)
                except:
                    sys.path.append(pydev_path)

                import pydevd
                pydevd.settrace('localhost', port=port, stdoutToServer=True, stderrToServer=True, suspend=suspend)


def get_path(directory='', file_name_with_extension=''):
    # type: (str, str) -> str
    return os.path.join(os.path.dirname(os.path.abspath(__file__)), directory, file_name_with_extension)


def read_json_from_file_as_dictionary(json_file_path):
    with open(json_file_path, 'rb') as content_file:
        complete_json = content_file.read()
        return json.loads(complete_json.decode("utf-8"))


class ExtendedEnum(Enum):

    @classmethod
    def list_values(cls):
        # type: () -> list
        return list(map(lambda enum: enum.value, cls))

    @classmethod
    def list_names(cls):
        # type: () -> list
        return list(map(lambda enum: enum.name, cls))

    @classmethod
    def list_enums_tuple(cls):
        # type: () -> list
        return list(map(lambda enum: (enum.name, enum.value), cls))

    @classmethod
    def get_name(cls, value):
        # type: (int) -> Union[str, None]
        if value not in cls.list_values():
            return None

        return next(str(enum.name) for enum in cls if enum.value == value)

    @classmethod
    def get_value(cls, name):
        # type: (str) -> Union[int, None]
        if name not in cls.list_names():
            return None

        return next(int(enum.value) for enum in cls if enum.name == name)



from typing import Dict, Optional, Union

from posot import OrderTaker, OrderTakerException
from sysactions import StopAction

from actions.models.mw_sys_errors import MwSysErrors


def create_order(
    pos_ot,             # type: OrderTaker
    pos_id,             # type: Union[int, str]
    price_list,         # type: str
    multi_order_id="",  # type: Optional[str]
    order_type="SALE",  # type: Optional[str]
    sale_type="",       # type: Optional[str]
    order_sub_type="",  # type: Optional[str]
):
    # type: (...) -> Dict

    try:
        return pos_ot.createOrder(
            posid=int(pos_id),
            pricelist=price_list,
            saletype=sale_type,
            multiorderid=multi_order_id,
            orderType=order_type,
            orderSubType=order_sub_type,
        )
    except OrderTakerException as ex:
        error_code_message, error_message = ex.message.split(";")
        error_code = error_code_message.split(":")[-1].strip()
        if error_code == MwSysErrors.SE_ORDER_IS_ALREADY_OPENED.value:
            raise StopAction()
        raise



import logging

from actions.products.get_products import getProducts, get_product_by_barcode, get_product_price_from_cache
from sysactions import get_model, get_podtype, show_messagebox
from systools import sys_log_exception

logger = logging.getLogger("PosActions")


def sell_product_by_barcode(pos_id, barcode_number, check_consistency=False, received_qty="1"):
    try:

        from posactions import doSale, doCompleteOption, doChangeQuantity
        if not barcode_number:
            return "False"

        logger.info("Barcode {0} ".format(barcode_number))
        if check_consistency and not _is_a_valid_ean13(barcode_number):
            ret = show_messagebox(pos_id, "$BARCODE_WRONG", buttons="$OK|$CANCEL")
            if ret == 0:
                return "Clear"
            return "False"
        model = get_model(pos_id)
        pod_type = get_podtype(model)
        if str(barcode_number).zfill(13).startswith('2'):
            plu, calc_qty = _ticket_from_scale(str(barcode_number), pos_id)
            if pod_type == 'TT':
                doChangeQuantity(int(pos_id), None, str(calc_qty))
                doSale(str(pos_id), '1.' + str(plu), str(calc_qty))
            else:
                doChangeQuantity(int(pos_id), None, str(calc_qty))
                doCompleteOption(str(pos_id), "1", plu, calc_qty)
        else:
            product_by_barcode = get_product_by_barcode(barcode_number)
            if product_by_barcode:
                if product_by_barcode["ruptured"]:
                    logger.info("Scanned barcode {0} product is ruptured!".format(barcode_number))
                    ret = show_messagebox(pos_id,
                                          "$BARCODE_PRODUCT_RUPTURED|{}".format(product_by_barcode["name"]),
                                          buttons="$YES|$NO")
                    if ret != 0:
                        return "Clear"

                if pod_type == 'TT':
                    doSale(str(pos_id), '1.' + str(product_by_barcode['plu']), received_qty)
                else:
                    doCompleteOption(str(pos_id), "1", product_by_barcode['plu'], received_qty)
            else:
                logger.info("Scanned barcode {0} not found on the product database!".format(barcode_number))
                ret = show_messagebox(pos_id, "$BARCODE_NOT_FOUND|{}".format(barcode_number), buttons="$OK|$CANCEL")
                if ret == 0:
                    return "Clear"
                return "False"
        return "True"
    except Exception as _:
        sys_log_exception("Error on [sell_product_by_barcode]")
        logger.exception("Error on [sell_product_by_barcode]")
        return "False"


def _is_a_valid_ean13(barcode):
    if len(str(barcode)) != 13:
        return False
    barcode = list(str(barcode).zfill(13))
    index = 0
    value = 0
    dv = barcode.pop()
    for x in barcode:
        value = value + (int(x) * ((index % 2) * 2 + 1))
        index += 1
    value2 = (((int(value / 10) + 1) * 10) - value) % 10
    if value2 != int(dv):
        return False
    return True


def _ticket_from_scale(barcode, pos_id):
    from posactions import scale_ticket_mode
    plu = int(barcode[1:7])
    param = int(barcode[7:12])
    if scale_ticket_mode == 'price':
        final_price = float(param) / 100
        price = float(get_product_price_from_cache(plu, pos_id))
        qtd = round((final_price / price), 3)
    else:
        qtd = round(float(param) / 1000, 3)
    return str(plu), qtd



from abc import ABCMeta, abstractmethod

from typing import List  # noqa

from ._Part import Part


class Report(object):
    __metaclass__ = ABCMeta

    key = "6910d546-252f-4b28-bc20-fae0aa8a2806"

    @abstractmethod
    def get_parts(self):
        # type: () -> List[Part]
        raise NotImplementedError()

    def get_width(self):
        # type: () -> int
        return 40



from typing import Dict, Optional

from messagebus import HttpRequestMessage, HttpResponseMessage
from messageprocessor import MessageProcessor
from messageprocessorutil import UuidMessageProcessor
from simplehttprouter import Route, SimpleHttpRouter

from ._ApiRequest import ApiRequest
from ._RouteNotMapped import RouteNotMapped


class HttpMessageProcessor(UuidMessageProcessor):
    def __init__(self, route_processor, event_name=None):
        # type: (Dict[Route, MessageProcessor], str) -> None
        super(HttpMessageProcessor, self).__init__(event_name)
        self.route_processor = route_processor
        self.router = SimpleHttpRouter(list(self.route_processor.keys()))

        self.processor = None
        self.route_name = None

    def get_processor_name(self):
        if self.route_name is not None:
            return self.route_name
        return super(HttpMessageProcessor, self).get_processor_name()

    def parse_data(self, request, event_type):
        # type: (HttpRequestMessage, Optional[str]) -> object
        try:
            route_param = Route(request.method, request.url)
            route = self.router.get_route(route_param)
            api_request = ApiRequest(
                request.method,
                route.route.path,
                route.path_parameters,
                request.query,
                request.http_headers,
                request.body)
            self.processor = self.route_processor.get(route.route)
            self.route_name = "{}${}${}".format(super(HttpMessageProcessor, self).get_processor_name(), route.route.method, route.route.path)
            return self.processor.parse_data(api_request, None)
        except AttributeError:
            raise RouteNotMapped(request.method, request.url)

    def call_business(self, obj):
        return self.processor.call_business(obj)

    def format_response(self, message_bus, message, event, input_obj, result):
        self.processor.format_response(message_bus, message, event, input_obj, result)

    def format_exception(self, message_bus, message, event, input_obj, exception):
        self.processor.format_exception(message_bus, message, event, input_obj, exception)

    def format_parse_exception(self, message_bus, message, event, data, exception):
        if isinstance(exception, RouteNotMapped):
            message_bus.reply_message(message, HttpResponseMessage(404))
        else:
            self.processor.format_parse_exception(message_bus, message, event, data, exception)




import json
from messagebus import \
    MessageBus, \
    Event, \
    Message, \
    DataType, \
    SwaggerMessage, \
    MessageBusException, \
    MessageBusExceptionType
from msgbus import MBEasyContext, MBMessage, MBException, FM_PARAM, FM_STRING, FM_XML
from syserrors import SE_TIMEOUT, SE_MB_RECEIVE


class MbContextMessageBus(MessageBus):
    def __init__(self, mbcontext):
        # type: (MBEasyContext) -> None
        self.mbcontext = mbcontext

    def publish_event(self, event):
        # type: (Event) -> Message
        data = event.data
        if data is None:
            data = ""

        if event.event_type is None:
            event.event_type = ""

        msg = self.mbcontext.MB_EasyEvtSend(subject=event.subject,
                                            type=event.event_type,
                                            xml=data,
                                            synchronous=False,
                                            sourceid=event.source,
                                            queue=None,
                                            timeout=event.timeout)
        return self._convert_mb_message_to_message(msg)

    def send_message(self, component, message):
        data = message.data
        if data is None:
            data = ""

        try:
            msg = self.mbcontext.MB_EasySendMessage(component, message.token, message.data_type, data, message.timeout)
            return self._convert_mb_message_to_message(msg)
        except MBException as ex:
            if ex.errorcode == SE_TIMEOUT:
                raise MessageBusException(MessageBusExceptionType.timeout)
            elif ex.errorcode == SE_MB_RECEIVE:
                raise MessageBusException(MessageBusExceptionType.receive_error)
            else:
                raise MessageBusException(MessageBusExceptionType.unknown)

    def subscribe(self, subject):
        if isinstance(subject, list):
            subject_to_subscribe = " ".join(subject)
        else:
            subject_to_subscribe = subject
        self.mbcontext.MB_EasyEvtSubscribe(subject_to_subscribe)

    def reply_message(self, message, reply_message):
        self._reply_mb_message(message.imp_message, reply_message)

    def reply_event(self, event, reply_message):
        self._reply_mb_message(event.imp_message, reply_message)

    def _reply_mb_message(self, msg, reply_message):
        # type: (MBMessage, Message) -> None
        msg.token = reply_message.token
        data = reply_message.data
        if reply_message.data is None:
            data = ""

        if isinstance(reply_message, SwaggerMessage):
            transformed_data = {
                "status": reply_message.status_code
            }

            if reply_message.body is not None:
                transformed_data["body"] = reply_message.body

            if reply_message.headers is not None:
                transformed_data["headers"] = reply_message.headers

            reply_message.data = transformed_data

        if isinstance(reply_message.data, dict):
            data = json.dumps(reply_message.data)

        self.mbcontext.MB_ReplyMessage(msg,
                                       format=reply_message.data_type,
                                       data=data)

    @staticmethod
    def _convert_mb_message_to_message(msg):
        # type: (MBMessage) -> Message
        data_format = msg.format
        data = msg.data
        if msg.format == FM_PARAM:
            if msg.data == "" or msg.data is None:
                data_format = DataType.empty
                data = None
            else:
                data_format = DataType.param
        elif msg.format == FM_XML:
            data_format = DataType.xml
        elif msg.format == FM_STRING:
            data_format = DataType.string

        return Message(msg.token, data_format, data)




class PaymentType(object):

    def __init__(
            self,
            name=None,          # type: str
            payment_type=None,   # type: int
            image_path=None     # type: str
    ):
        # type: (...) -> None

        self.name = name
        self.payment_type = payment_type
        self.image_path = image_path



from _EscPosImage import EscPosImage
from _QrCodeGenerator import QrCodeGenerator
import six


GS = b'\x1d'


class EscPosQrCodeGenerator(object):
    @staticmethod
    def generate_qr_code(content):
        # type: (unicode) -> str
        img = QrCodeGenerator.generate_qr_code(content)
        return EscPosQrCodeGenerator._print_image(EscPosImage(img))

    @staticmethod
    def _print_image(esc_pos_image):
        # type: (EscPosImage) -> str
        density_byte = 0
        header = GS + \
                 b"v0" + \
                 six.int2byte(density_byte) + \
                 EscPosQrCodeGenerator._int_low_high(esc_pos_image.width_bytes, 2) + \
                 EscPosQrCodeGenerator._int_low_high(esc_pos_image.height, 2)
        return header + esc_pos_image.to_raster_format()

    @staticmethod
    def _int_low_high(inp_number, out_bytes):
        """ Generate multiple bytes for a number: In lower and higher parts, or more parts as needed.

        :param inp_number: Input number
        :param out_bytes: The number of bytes to output (1 - 4).
        """
        max_input = (256 << (out_bytes * 8) - 1)
        if not 1 <= out_bytes <= 4:
            raise ValueError("Can only output 1-4 byes")
        if not 0 <= inp_number <= max_input:
            raise ValueError("Number too large. Can only output up to {0} in {1} byes".format(max_input, out_bytes))
        outp = b''
        for _ in range(0, out_bytes):
            outp += six.int2byte(inp_number % 256)
            inp_number //= 256
        return outp



from ._AddBenefitProcessor import AddBenefitProcessor
from ._GetBenefitsProcessor import GetBenefitsProcessor
from ._RemoveBenefitProcessor import RemoveBenefitProcessor
from ._OnOrderReopenProcessor import OnOrderReopenProcessor
from ._OnSaleItemProcessor import OnSaleItemProcessor
from ._OnOrderTotalProcessor import OnOrderTotalProcessor
from ._OnTableOpenProcessor import OnTableOpenProcessor
from ._OnTabOpenProcessor import OnTabOpenProcessor



"""Response classes used by urllib.

The base class, addbase, defines a minimal file-like interface,
including read() and readline().  The typical response object is an
addinfourl instance, which defines an info() method that returns
headers and a geturl() method that returns the url.
"""
from __future__ import absolute_import, division, unicode_literals
from future.builtins import object

class addbase(object):
    """Base class for addinfo and addclosehook."""

    # XXX Add a method to expose the timeout on the underlying socket?

    def __init__(self, fp):
        # TODO(jhylton): Is there a better way to delegate using io?
        self.fp = fp
        self.read = self.fp.read
        self.readline = self.fp.readline
        # TODO(jhylton): Make sure an object with readlines() is also iterable
        if hasattr(self.fp, "readlines"):
            self.readlines = self.fp.readlines
        if hasattr(self.fp, "fileno"):
            self.fileno = self.fp.fileno
        else:
            self.fileno = lambda: None

    def __iter__(self):
        # Assigning `__iter__` to the instance doesn't work as intended
        # because the iter builtin does something like `cls.__iter__(obj)`
        # and thus fails to find the _bound_ method `obj.__iter__`.
        # Returning just `self.fp` works for built-in file objects but
        # might not work for general file-like objects.
        return iter(self.fp)

    def __repr__(self):
        return '<%s at %r whose fp = %r>' % (self.__class__.__name__,
                                             id(self), self.fp)

    def close(self):
        if self.fp:
            self.fp.close()
        self.fp = None
        self.read = None
        self.readline = None
        self.readlines = None
        self.fileno = None
        self.__iter__ = None
        self.__next__ = None

    def __enter__(self):
        if self.fp is None:
            raise ValueError("I/O operation on closed file")
        return self

    def __exit__(self, type, value, traceback):
        self.close()

class addclosehook(addbase):
    """Class to add a close hook to an open file."""

    def __init__(self, fp, closehook, *hookargs):
        addbase.__init__(self, fp)
        self.closehook = closehook
        self.hookargs = hookargs

    def close(self):
        if self.closehook:
            self.closehook(*self.hookargs)
            self.closehook = None
            self.hookargs = None
        addbase.close(self)

class addinfo(addbase):
    """class to add an info() method to an open file."""

    def __init__(self, fp, headers):
        addbase.__init__(self, fp)
        self.headers = headers

    def info(self):
        return self.headers

class addinfourl(addbase):
    """class to add info() and geturl() methods to an open file."""

    def __init__(self, fp, headers, url, code=None):
        addbase.__init__(self, fp)
        self.headers = headers
        self.url = url
        self.code = code

    def info(self):
        return self.headers

    def getcode(self):
        return self.code

    def geturl(self):
        return self.url

del absolute_import, division, unicode_literals, object



# encoding: utf-8
from escposformatter import DefaultEscPosFormatter
from mwposreport import ReporterFactory

from reportsdata.deliveryreport import DeliveryGenerator, DeliveryPresenter
from deliveryreport.repository import DeliveryOrderRepository, DeliveryProductRepository


class ReportGenerator(object):
    @staticmethod
    def generate(mb_context, pos_id, order_id, pos_list):
        if not pos_list:
            pos_list = [pos_id]
        else:
            pos_list = pos_list.split(",")
            
        order_repository = DeliveryOrderRepository(mb_context, pos_list)
        product_repository = DeliveryProductRepository(mb_context)
        report = ReporterFactory.build_reporter(pos_id,
                                                DeliveryGenerator(order_repository, product_repository, order_id),
                                                DeliveryPresenter(56)).create_base_report()
        return DefaultEscPosFormatter().format(report.get_parts())



import os
import pytz
import re
import warnings

from tzlocal import utils

_cache_tz = None


def _tz_from_env(tzenv):
    if tzenv[0] == ':':
        tzenv = tzenv[1:]

    # TZ specifies a file
    if os.path.isabs(tzenv) and os.path.exists(tzenv):
        with open(tzenv, 'rb') as tzfile:
            return pytz.tzfile.build_tzinfo('local', tzfile)

    # TZ specifies a zoneinfo zone.
    try:
        tz = pytz.timezone(tzenv)
        # That worked, so we return this:
        return tz
    except pytz.UnknownTimeZoneError:
        raise pytz.UnknownTimeZoneError(
            "tzlocal() does not support non-zoneinfo timezones like %s. \n"
            "Please use a timezone in the form of Continent/City")


def _try_tz_from_env():
    tzenv = os.environ.get('TZ')
    if tzenv:
        try:
            return _tz_from_env(tzenv)
        except pytz.UnknownTimeZoneError:
            pass


def _get_localzone(_root='/'):
    """Tries to find the local timezone configuration.

    This method prefers finding the timezone name and passing that to pytz,
    over passing in the localtime file, as in the later case the zoneinfo
    name is unknown.

    The parameter _root makes the function look for files like /etc/localtime
    beneath the _root directory. This is primarily used by the tests.
    In normal usage you call the function without parameters."""

    tzenv = _try_tz_from_env()
    if tzenv:
        return tzenv

    # Are we under Termux on Android?
    if os.path.exists('/system/bin/getprop'):
        import subprocess
        androidtz = subprocess.check_output(['getprop', 'persist.sys.timezone']).strip().decode()
        return pytz.timezone(androidtz)

    # Now look for distribution specific configuration files
    # that contain the timezone name.
    for configfile in ('etc/timezone', 'var/db/zoneinfo'):
        tzpath = os.path.join(_root, configfile)
        try:
            with open(tzpath, 'rb') as tzfile:
                data = tzfile.read()

                # Issue #3 was that /etc/timezone was a zoneinfo file.
                # That's a misconfiguration, but we need to handle it gracefully:
                if data[:5] == b'TZif2':
                    continue

                etctz = data.strip().decode()
                if not etctz:
                    # Empty file, skip
                    continue
                for etctz in data.decode().splitlines():
                    # Get rid of host definitions and comments:
                    if ' ' in etctz:
                        etctz, dummy = etctz.split(' ', 1)
                    if '#' in etctz:
                        etctz, dummy = etctz.split('#', 1)
                    if not etctz:
                        continue
                    tz = pytz.timezone(etctz.replace(' ', '_'))
                    if _root == '/':
                        # We are using a file in etc to name the timezone.
                        # Verify that the timezone specified there is actually used:
                        utils.assert_tz_offset(tz)
                    return tz

        except IOError:
            # File doesn't exist or is a directory
            continue

    # CentOS has a ZONE setting in /etc/sysconfig/clock,
    # OpenSUSE has a TIMEZONE setting in /etc/sysconfig/clock and
    # Gentoo has a TIMEZONE setting in /etc/conf.d/clock
    # We look through these files for a timezone:

    zone_re = re.compile(r'\s*ZONE\s*=\s*\"')
    timezone_re = re.compile(r'\s*TIMEZONE\s*=\s*\"')
    end_re = re.compile('\"')

    for filename in ('etc/sysconfig/clock', 'etc/conf.d/clock'):
        tzpath = os.path.join(_root, filename)
        try:
            with open(tzpath, 'rt') as tzfile:
                data = tzfile.readlines()

            for line in data:
                # Look for the ZONE= setting.
                match = zone_re.match(line)
                if match is None:
                    # No ZONE= setting. Look for the TIMEZONE= setting.
                    match = timezone_re.match(line)
                if match is not None:
                    # Some setting existed
                    line = line[match.end():]
                    etctz = line[:end_re.search(line).start()]

                    # We found a timezone
                    tz = pytz.timezone(etctz.replace(' ', '_'))
                    if _root == '/':
                        # We are using a file in etc to name the timezone.
                        # Verify that the timezone specified there is actually used:
                        utils.assert_tz_offset(tz)
                    return tz

        except IOError:
            # File doesn't exist or is a directory
            continue

    # systemd distributions use symlinks that include the zone name,
    # see manpage of localtime(5) and timedatectl(1)
    tzpath = os.path.join(_root, 'etc/localtime')
    if os.path.exists(tzpath) and os.path.islink(tzpath):
        tzpath = os.path.realpath(tzpath)
        start = tzpath.find("/")+1
        while start != 0:
            tzpath = tzpath[start:]
            try:
                return pytz.timezone(tzpath)
            except pytz.UnknownTimeZoneError:
                pass
            start = tzpath.find("/")+1

    # No explicit setting existed. Use localtime
    for filename in ('etc/localtime', 'usr/local/etc/localtime'):
        tzpath = os.path.join(_root, filename)

        if not os.path.exists(tzpath):
            continue
        with open(tzpath, 'rb') as tzfile:
            return pytz.tzfile.build_tzinfo('local', tzfile)

    warnings.warn('Can not find any timezone configuration, defaulting to UTC.')
    return pytz.utc

def get_localzone():
    """Get the computers configured local timezone, if any."""
    global _cache_tz
    if _cache_tz is None:
        _cache_tz = _get_localzone()

    return _cache_tz


def reload_localzone():
    """Reload the cached localzone. You need to call this if the timezone has changed."""
    global _cache_tz
    _cache_tz = _get_localzone()
    return _cache_tz



from decimal import Decimal

import persistence
from application.repository import DrawerRepository
from sysactions import show_info_message
from systools import sys_log_exception

from .. import logger, mb_context


def get_drawer_amount(pos_id, period, session):
    logger.debug("--- get_drawer_amount START ---")
    conn = None
    try:
        conn = persistence.Driver().open(mb_context, dbname=str(pos_id))
        conn.transaction_start()
        # set the period
        logger.debug("--- get_drawer_amount DELETE ReportsPeriod ---")
        conn.query("DELETE FROM temp.ReportsPeriod")
        logger.debug("--- get_drawer_amount INSERT ReportsPeriod ---")
        conn.query("INSERT INTO temp.ReportsPeriod(StartPeriod,EndPeriod) VALUES(%s,%s)" % (period, period))
        logger.debug("--- get_drawer_amount SELECT Transfer ---")
        cursor = conn.select("""SELECT T.Period AS Period,
           T.SessionId AS SessionId,
           SUM(CASE WHEN T.Type=1 THEN 1 ELSE 0 END) AS InitialFloatQty,
           tdsum(CASE WHEN T.Type=1 THEN T.Amount ELSE '0.00' END) AS InitialFloatAmount,
           SUM(CASE WHEN T.Type=2 THEN 1 ELSE 0 END) AS SkimQty,
           tdsum(CASE WHEN T.Type=2 THEN T.Amount ELSE '0.00' END) AS SkimAmount,
           SUM(CASE WHEN T.Type=3 THEN 1 ELSE 0 END) AS TransferInQty,
           tdsum(CASE WHEN T.Type=3 THEN T.Amount ELSE '0.00' END) AS TransferInAmount,
           SUM(CASE WHEN T.Type=4 THEN 1 ELSE 0 END) AS TransferOutQty,
           tdsum(CASE WHEN T.Type=4 THEN T.Amount ELSE '0.00' END) AS TransferOutAmount
            FROM
                Transfer T
            WHERE T.Period = '%s'
            AND T.SessionId = '%s'
            GROUP BY Period, SessionId""" % (period, session))

        row = cursor.get_row(0)
        if row is not None:
            initial_float = Decimal(row.get_entry("InitialFloatAmount") or 0)
            transfer_in_amount = Decimal(row.get_entry("TransferInAmount") or 0)
            skim_amount = Decimal(row.get_entry("SkimAmount") or 0)
            transfer_out_amount = Decimal(row.get_entry("TransferOutAmount") or 0)
        else:
            initial_float = 0
            transfer_in_amount = 0
            skim_amount = 0
            transfer_out_amount = 0

        logger.debug("--- get_drawer_amount SELECT Orders ---")
        drawer_repository = DrawerRepository(mb_context)
        all_cash_gross_amount, all_refund_cash_amount = drawer_repository.get_user_sum_cash_values(period, session)

        logger.debug("--- get_drawer_amount SELECT OrderCustomProperties inner Orders---")
        cursor = conn.select("""SELECT COALESCE(SUM(ocp.Value), 0) AS DonatedAmount FROM OrderCustomProperties ocp 
                                INNER JOIN Orders o 
                                ON o.OrderId = ocp.OrderId 
                                WHERE ocp.Key = 'DONATION_VALUE' 
                                AND o.BusinessPeriod = '%s' 
                                AND o.SessionId = '%s'""" % (period, session))
        row = cursor.get_row(0)
        donated_amount = Decimal(row.get_entry("DonatedAmount") or 0)
        sum_values = initial_float + all_cash_gross_amount + transfer_in_amount + donated_amount
        subtract_values = all_refund_cash_amount + skim_amount + transfer_out_amount
        drawer_amount = sum_values - subtract_values
        drawer_amount = round(drawer_amount, 2)
    except Exception as _:
        logger.exception("--- get_drawer_amount ---")
        sys_log_exception("get_drawer_amount Error")
        show_info_message(pos_id, "$OPERATION_FAILED", msgtype="error")
        return
    finally:
        if conn:
            conn.close()
    logger.debug("--- get_drawer_amount END ---")
    return drawer_amount



import os
import sys

import cfgtools
from helper import config_logger, import_pydevd
from messagehandler import MessageHandler
from msgbus import MBEasyContext

from eventhandler import FingerPrintReaderEventHandler, FingerPrintReaderCallback
from fingerprintreader import DigitalPersonaFingerPrintReader, DatabaseEnrolledUsersRepository

REQUIRED_SERVICES = ""


def main():
    import_pydevd(os.environ["LOADERCFG"], 9128)

    config_logger(os.environ["LOADERCFG"], 'FingerPrintReader')

    config = cfgtools.read(os.environ["LOADERCFG"])
    mbcontext = MBEasyContext("FingerPrintReader")

    module_directory = config.find_value("FingerPrinterReader.ModuleDirectory")
    number = int(config.find_value("FingerPrinterReader.Number"))
    enrolled_users_cache_expiration = int(config.find_value("FingerPrinterReader.EnrolledUsersCacheExpiration"))
    threshold = int(config.find_value("FingerPrinterReader.Threshold"))
    finger_print_timeout = int(config.find_value("FingerPrinterReader.FingerPrintTimeout"))

    finger_print_reader_callback = FingerPrintReaderCallback(mbcontext)
    enrolled_users_repository = DatabaseEnrolledUsersRepository(mbcontext, enrolled_users_cache_expiration)
    finger_print_reader = DigitalPersonaFingerPrintReader(enrolled_users_repository, finger_print_reader_callback, threshold, finger_print_timeout, module_directory)
    finger_print_event_handler = FingerPrintReaderEventHandler(mbcontext, finger_print_reader)

    MessageHandler(mbcontext, "FingerPrintReader{0}".format(number), "FingerPrintReader", REQUIRED_SERVICES, finger_print_event_handler).handle_events()



# -*- coding: utf-8 -*-

from requests import exceptions
from nfcecanceler import NfceCancelerResponseParser, NfceCancelerRequestBuilder, NfceCancelerResponse
from nfcebuilder.nfceutil import NfceRequest
import logging
import os

logger = logging.getLogger("FiscalWrapper")


class NfceCanceler(object):
    def __init__(self, nfce_canceler_request_builder, nfce_canceler_response_parser, nfce_request, url_cancelamento, dir_enviados, versao_ws):
        # type: (NfceCancelerRequestBuilder, NfceCancelerResponseParser, NfceRequest, str, str, int) -> None
        self.nfce_canceler_request_builder = nfce_canceler_request_builder
        self.nfce_canceler_response_parser = nfce_canceler_response_parser
        self.dir_enviados = dir_enviados
        self.nfce_request = nfce_request
        self.url_cancelamento = url_cancelamento
        self.versao_ws = versao_ws
        if versao_ws in (1, 3):
            self.soap_act = "http://www.portalfiscal.inf.br/nfe/wsdl/RecepcaoEvento/nfeRecepcaoEvento"
        else:
            self.soap_act = "http://www.portalfiscal.inf.br/nfe/wsdl/NFeRecepcaoEvento4/nfeRecepcaoEvento"

    def cancel_order(self, order_response, justificativa, order_id):
        cancel_request_xml = self.nfce_canceler_request_builder.build_request(order_response, justificativa, self.versao_ws)

        try:
            response = self.nfce_request.envia_nfce(cancel_request_xml, self.url_cancelamento, soap_action=self.soap_act)
            #logger.info("disable_fiscal_number, request: {} response code: {}: text{}".format(cancel_request_xml, response.status_code, response.text))
        except exceptions.RequestException:
            return False, "Problemas de Conexao"
        except Exception as other_e:
            return False, str(other_e)
        else:
            nfce_canceler_response = self.nfce_canceler_response_parser.parse_response(response.text, self.versao_ws)
            if nfce_canceler_response.c_stat not in (NfceCancelerResponse.Canceled, NfceCancelerResponse.AlreadyCanceled):
                return False, 'Invalid cStat {0}'.format(nfce_canceler_response.c_stat)
            else:
                if nfce_canceler_response.c_stat == NfceCancelerResponse.Canceled:
                    idx_start = cancel_request_xml.find("<chNFe>") + 7
                    idx_final = cancel_request_xml.find("</chNFe>")
                    chave = cancel_request_xml[idx_start:idx_final]
                    serie_nota = chave[22:25]
                    numero_nota = chave[25:34].zfill(9)
                    idx_start = cancel_request_xml.find("<dhEvento>") + 10
                    idx_final = cancel_request_xml.find("</dhEvento>")
                    data_emissao = cancel_request_xml[idx_start:idx_final]
                    dir_arquivo = os.path.join(data_emissao[0:4], data_emissao[5:7], data_emissao[8:10])
                    name_template = "{}_{}_{}_nfe_cancelamento_NFe{}.xml"
                    file_name = name_template.format(serie_nota, numero_nota, order_id.zfill(9), chave)
                    file_path = os.path.join(self.dir_enviados, "Enviados", dir_arquivo, file_name)
                    with open(file_path, "w+") as nfe_cancel_file:
                        nfe_cancel_file.write(response.text)

                    return True, "Venda cancelada com sucesso"
                elif nfce_canceler_response.c_stat == NfceCancelerResponse.AlreadyCanceled:
                    return True, "Venda já cancelada anteriormente"
                else:
                    return False, "Erro ao tentar cancelar venda"



import base64

from messagebus import Message, DefaultToken, DataType, MessageBus, TokenPriority, TokenCreator, Event
from messageprocessorutil import UuidMessageProcessor

TK_POS_MODELSETCUSTOM = TokenCreator.create_token(TokenPriority.high, "5", "4")


class PrnPrintProcessor(UuidMessageProcessor):
    def __init__(self, pos_id, message_bus):
        # type: (int, MessageBus) -> None
        super(PrnPrintProcessor, self).__init__()

        self.pos_id = pos_id
        self.message_bus = message_bus

    def parse_data(self, data):
        return data

    def call_business(self, print_data):
        event_xml = """<Event subject="POS{}" type="MOBILE">
                           <MOBILE_PRINT>
                               {}
                           </MOBILE_PRINT>
                       </Event>""".format(self.pos_id, base64.b64encode(print_data))

        return Event("POS{}".format(self.pos_id), "MOBILE", event_xml)

    def format_response(self, message_bus, message, event, input_obj, result):
        message_bus.publish_event(result)
        message_bus.reply_message(message, Message(DefaultToken.TK_SYS_ACK, DataType.param))

    def format_exception(self, message_bus, message, event, input_obj, exception):
        message_bus.reply_message(message, Message(DefaultToken.TK_SYS_NAK, DataType.param, str(exception)))

    def format_parse_exception(self, message_bus, message, event, data, exception):
        message_bus.reply_message(message, Message(DefaultToken.TK_SYS_NAK, DataType.param, str(exception)))



from _PickListHeaderDto import PickListHeaderDto  # noqa
from _PickListBodyDto import PickListBodyDto  # noqa


class PickListDto(object):
    def __init__(self, pick_list_header, pick_list_body, pod_type, sale_type, pos_id):
        # type: (PickListHeaderDto, PickListBodyDto, unicode, int) -> ()
        self.pick_list_header = pick_list_header
        self.pick_list_body = pick_list_body
        self.pod_type = pod_type
        self.sale_type = sale_type
        self.pos_id = pos_id



import logging
from logging import Logger

from cfgtools import read
from helper import config_logger
from model.config import Config, PosMap
from systools import sys_parsefilepath
from typing import List


class Configurator(object):

    def __init__(self, loader_path):
        # type: (str) -> None

        self.config = Config()
        self.config.loader_path = loader_path
        self.reader = read(loader_path)

    def read_loader_configs(self):
        # type: () -> Config

        group = "InternalOrder."

        self.config.component_name = str(self.reader.find_value("Identification.ComponentName"))
        self.config.required_services = "|".join(self.reader.find_values(group + "RequiredServices") or [])
        self.config.host = str(self.reader.find_value(group + "FlaskConfiguration.Host"))
        self.config.port = str(self.reader.find_value(group + "FlaskConfiguration.Port"))
        self.config.cors_origin_hosts = self.reader.find_values(group + "FlaskConfiguration.AllowedOriginHosts")

        if self.config.component_name == "InternalOrderMessageBus":
            return self.config

        self.config.allowed_payment_methods = map(int, self.reader.find_values(group + "AllowedPaymentMethods") or [])
        self.config.available_sale_types = "|".join(self.reader.find_values(group + "AvailableSaleTypes") or [])
        self.config.ruptured_items_cache_time = int(self.reader.find_value(group + "RupturedItemsCacheTime") or 10)
        self.config.api_key_mapping = self.build_pos_map(group)
        self.config.images_path = sys_parsefilepath(self.reader.find_value(group + "ImagesPath"))
        self.config.totem_config_path = sys_parsefilepath(self.reader.find_value(group + "TotemConfigPath"))
        self.config.up_sell_part_codes_path = sys_parsefilepath(self.reader.find_value(group + "UpSellPartCodes"))

        return self.config

    def configure_logger(self):
        # type: () -> Logger

        config_logger(self.config.loader_path, self.config.component_name, root_level=logging.DEBUG)
        return logging.getLogger(self.config.component_name)

    def build_pos_map(self, group):
        # type: (str) -> List[PosMap]

        mapped_pos = []
        to_map_groups = self.reader.find_group(group + "PosMap")

        for group in to_map_groups.groups:
            pos_map = PosMap()
            pos_map.pos_id = int(group.name)
            pos_map.api_key = group.find_value("ApiKey")
            pos_map.abstract_pos_id = int(group.find_value("AbstractPosId"))
            pos_map.user_id = int(group.find_value("UserId"))
            pos_map.user_password = int(group.find_value("UserPassword"))

            mapped_pos.append(pos_map)

        return mapped_pos



from logging import Logger

from model.config import Config
from model.error_type import ExternalServicesErrorsType
from model.exception import ExternalServiceErrors
from model.repository import PosRepositoryModel
from interactor.route.store.get_status.dto import (
    GetStoreResponse,
    GetStoreRequest
)
from validation_util import validate_api_key


class GetStoreInteractor(object):
    def __init__(self, logger, config, pos_repository):
        # type: (Logger, Config, PosRepositoryModel) -> None

        self.logger = logger
        self.config = config
        self.pos_repository = pos_repository

    def execute(self, request):
        # type: (GetStoreRequest) -> GetStoreResponse

        pos_id = validate_api_key(request, self.config.api_key_mapping)

        status = None
        if self.config.nf_type == 'NFCE':
            status = self.pos_repository.get_sefaz_status()
        elif self.config.nf_type == 'SAT':
            status = self.pos_repository.get_sat_status(pos_id)

        if not status:
            raise ExternalServiceErrors(ExternalServicesErrorsType.UnableToGetStatus)

        menu_id = self.pos_repository.get_catalog_version()
        if not menu_id:
            raise ExternalServiceErrors(ExternalServicesErrorsType.UnableToGetMenuId)

        return GetStoreResponse(status, menu_id)



# -*- coding: utf-8 -*-

from modelcommon.model import SitefResponse
from typing import (
    Any,
    Union
)

from _CapptaProcessor import CapptaProcessor
from model import (
    CancelPaymentOrderData,
    ConfirmFinishPaymentData,
    TestConnectionData
)


class CapptaDebitPaymentProcessor(object):
    def __init__(self, message_bus, dialog_manager, config, logger):
        self.cappta_processor = CapptaProcessor(message_bus, dialog_manager, config, logger)

    def connection_status(
        self,
        test_connection_data    # type: TestConnectionData
    ):
        # type: (...) -> bool

        return self.cappta_processor.connection_status(test_connection_data=test_connection_data)

    def add_payment(self, start_payment_data):
        return self.cappta_processor.add_payment(start_payment_data)

    def cancel_payment_order(
        self,
        cancel_payment_order_data   # type: CancelPaymentOrderData
    ):
        # type: (...) -> Any[Union[SitefResponse, str]]

        return self.cappta_processor.cancel_payment_order(cancel_payment_order_data=cancel_payment_order_data)

    def cancel_payment(self):
        return self.cappta_processor.cancel_payment()

    def finish_payment(self, finish_payment_data):
        self.cappta_processor.finish_payment(finish_payment_data)

    def confirm_finish_payment(
        self,
        confirm_finish_payment_data     # type: ConfirmFinishPaymentData
    ):
        # type: (...) -> None

        self.cappta_processor.confirm_finish_payment(confirm_finish_payment_data=confirm_finish_payment_data)

    def get_name(self):
        return self.cappta_processor.get_name()

    def stop(self):
        self.cappta_processor.stop()



from _CancelPaymentProcessor import CancelPaymentProcessor
from _CancelPaymentOrderProcessor import CancelPaymentOrderProcessor
from _ConfirmFinishPaymentProcessor import ConfirmFinishPaymentProcessor
from _DialogResponseProcessor import DialogResponseProcessor
from _FinishPaymentProcessor import FinishPaymentProcessor
from _GetElectronicTypesProcessor import GetElectronicTypesProcessor
from _StartPaymentProcessor import StartPaymentProcessor
from _TestConnectionProcessor import TestConnectionProcessor



import json

from sysactions import (
    action,
    show_any_dialog,
    show_messagebox,
    StopAction
)

from get_products import get_daily_dishes
from .. import logger


@action
def open_daily_dishes(
    pos_id  # type: str
):
    # type: (...) -> None

    try:
        available_daily_dishes = get_daily_dishes()
        available_dishes = available_daily_dishes["available"]
        quantity_set_dishes = available_daily_dishes["quantity_set"]
    except Exception as e:
        logger.exception('[rupture_items] Exception {}'.format(e))
        show_messagebox(
            posid=pos_id,
            message="$ERROR_GETTING_DATA",
            title="$INFORMATION",
            buttons="$EXIT"
        )
        raise StopAction()

    if not (available_daily_dishes or quantity_set_dishes):
        show_messagebox(pos_id, "$NO_RUPTURE_ITEMS")
        return

    dialog_contents = {
        "type": "daily_dishes",
        "enabledIngredients": available_dishes,
        "disabledIngredients": quantity_set_dishes
    }

    info_text = json.dumps(dialog_contents)
    show_any_dialog(
        posid=pos_id,
        dlgtype="rupture",
        message="",
        title="$MANAGE_DAILY_DISHES",
        infotext=info_text,
        buttons="",
        icon="",
        timeout=6000000,
        mask="",
        defvalue="",
        minvalue="",
        maxvalue="",
        contents="",
        asynch=False
    )



from datetime import datetime


class TableReportHeaderDto(object):

    def __init__(self):
        self.table_id = None  # type: str
        self.table_name = None  # type: str
        self.service_seats = None  # type: int
        self.start_date = None  # type: datetime
        self.totaled_date = None  # type: datetime
        self.user_id = None  # type: str
        self.user_name = None  # type: str
        self.order_id = None  # type: str



# -*- coding: utf-8 -*-

import sysactions
from systools import sys_log_exception

from .. import logger
from .. import mb_context


@sysactions.action
def manual_delivery_confirmation(pos_id, order_id):
    try:        
        mb_context.MB_EasyEvtSend("ManualConfirmDeliveryOrder", type="", xml=order_id, sourceid=int(pos_id))

    except Exception as e:
        sys_log_exception("Error ManualConfirmDeliveryOrder")
        logger.error(e)
        return False



from messagebus import MessageBus, Message


class GetReplyMessageBus(MessageBus):
    def __init__(self):
        self.reply = None

    def publish_event(self, event):
        pass

    def send_message(self, component, message):
        pass

    def subscribe(self, subscriptions):
        pass

    def reply_message(self, message, reply_message):
        self.reply = reply_message

    def reply_event(self, event, reply_message):
        self.reply = reply_message

    def get_reply(self):
        # type: () -> Message
        return self.reply



from abc import ABCMeta, abstractmethod
from typing import Optional, Dict

from ._Route import Route


class HttpRouter(object):
    __meta__ = ABCMeta

    @abstractmethod
    def get_route(self, route):
        # type: (Route) -> Optional[Route]
        raise NotImplementedError()

    @abstractmethod
    def get_path_parameters(self, route):
        # type: (Route) -> Dict[str, str]
        raise NotImplementedError()



from typing import Union


class TableOrderSaleLine(object):

    def __init__(self, product_name, price, qty, default_qty, item_type, seat, level, comment, line_number,
                 service_period):
        # type: (str, float, float, Union[int, None], str,  Union[int, None], int, Union[str, None], int, Union[str, None]) -> None
        self.product_name = product_name
        self.price = price
        self.qty = qty
        self.default_qty = default_qty
        self.item_type = item_type
        self.seat = seat
        self.level = level
        self.comment = comment
        self.line_number = line_number
        self.service_period = service_period



from logging import Logger

from production.box import ProductionBox
from production.command import ProductionCommandProcessor
from production.manager import PublishScheduler, OrderChanger, StatisticUpdater
from production.repository import ProductRepository
from production.treebuilder import BoxConfiguration
from production.treebuilder.boxfactory.builder import OrderStateFilterBoxBuilder, OrderSubTypeFilterBoxBuilder, \
    JitItemFilterBoxBuilder, HideOptionBoxBuilder, AllDoneBlinkBoxBuilder, ComboDoneWhenAllDoneBoxBuilder, \
    OrderSequencerBoxBuilder, RemoveDoingWhenDoneBoxBuilder, OrderStateTimerBoxBuilder, \
    FromOrderStateFilterBoxBuilder, CustomParamCourseBoxBuilder, CustomParamPrepTimeBoxBuilder, \
    ShowOnKitchenFilterBoxBuilder, NoJitAddTagBoxBuilder, ItemPropertiesFilterBoxBuilder, AllServedItemsBoxBuilder, \
    SendMomentTimerBuilder, ConditionBoxBuilder, CFHHiderBoxBuilder, CustomPropertyFilterBoxBuilder, \
    ComboFilterBoxBuilder, DistributorBoxBuilder, CustomParamPointerBoxBuilder, OrderRoundRobinBoxBuilder, \
    RemoveDeletedItemsBoxBuilder, FilterDeletedItemsBeforeFirstStateBoxBuilder, OrderSpreaderBoxBuilder, \
    OrderStampBoxBuilder, OrderMergerBoxBuilder, RecalledOrderStampBoxBuilder, ComboServedWhenAllServedBoxBuilder, \
    FilterOrderIfNotAllItemsHaveTagBoxBuilder, AllItemsDoneBoxBuilder, RemoveReprocessedOrdersBoxBuilder, \
    ProductCodeFilterBoxBuilder, OrderProductCategoryFilterBoxBuilder, PodTypeFilterBoxBuilder, \
    SaleTypeFilterBoxBuilder, SkipCourseBoxBuilder, CollapseSameItemsBoxBuilder, StampItemsViewBoxBuilder, \
    ItemSequencerBoxBuilder, StatisticBoxBuilder, AddTimeTagBoxBuilder, ProdStateFilterBoxBuilder, \
    CustomPropertyValueFilterBoxBuilder, ItemSorterBoxBuilder, DisplayTimeFilterBoxBuilder, ProdStateTimerBoxBuilder, \
    DecreaseTimeTagBoxBuilder, AllItemHasAtLeastOneTagFilterBoxBuilder, FilterOrderIfAllItemsHaveTagBoxBuilder, \
    FilterOrderIfAtLeastOneItemHasAtLeastOneTagFilterBoxBuilder, ComboTaggedWhenAllHasTagBoxBuilder, SorterBoxBuilder, \
    TagItemsWhenProdStateBoxBuilder, FilterDefaultQuantityIngredientsBoxBuilder, JitAddTagBoxBuilder, \
    ReprocessOrdersBoxBuilder, JitSequencerItemFilterBoxBuilder, SequencerTagItemBoxBuilder, OrderSplitterBoxBuilder, \
    OrderPropertiesFilterBoxBuilder

from production.view import ProductionView
from typing import Dict

from .builder import TagFilterBoxBuilder, ViewBoxBuilder


class ProductionBoxFactory(object):
    def __init__(self, command_processor, publish_scheduler, order_changer, statistic_updater, product_repository, loggers):
        # type: (ProductionCommandProcessor, PublishScheduler, OrderChanger, StatisticUpdater, ProductRepository, Dict[unicode, Logger]) -> None  # noqa
        self.command_processor = command_processor
        self.publish_scheduler = publish_scheduler
        self.order_changer = order_changer
        self.product_repository = product_repository
        self.loggers = loggers

        self.builders = {
            u"TagFilterBox": TagFilterBoxBuilder(loggers),
            u"OrderStateFilterBox": OrderStateFilterBoxBuilder(product_repository, loggers),
            u"OrderSubTypeFilterBox": OrderSubTypeFilterBoxBuilder(product_repository, loggers),
            u"JitItemFilterBox": JitItemFilterBoxBuilder(product_repository, loggers),
            u"HideOptionBox": HideOptionBoxBuilder(product_repository, loggers),
            u"ComboDoneWhenAllDoneBox": ComboDoneWhenAllDoneBoxBuilder(loggers),
            u"ComboServedWhenAllServedBox": ComboServedWhenAllServedBoxBuilder(loggers),
            u"AllDoneBlinkBox": AllDoneBlinkBoxBuilder(loggers),
            u"OrderSequencerBox": OrderSequencerBoxBuilder(product_repository, loggers),
            u"RemoveDoingWhenDoneBox": RemoveDoingWhenDoneBoxBuilder(loggers),
            u"OrderStateTimerBox": OrderStateTimerBoxBuilder(loggers),
            u"FromOrderStateFilterBox": FromOrderStateFilterBoxBuilder(product_repository, loggers),
            u"CustomParamCourseBox": CustomParamCourseBoxBuilder(product_repository, publish_scheduler, loggers),
            u"CFHHiderBox": CFHHiderBoxBuilder(product_repository, loggers),
            u"ComboFilterBox": ComboFilterBoxBuilder(loggers),
            u"DistributorBox": DistributorBoxBuilder(loggers),
            u"CustomParamPrepTimeBox": CustomParamPrepTimeBoxBuilder(product_repository, publish_scheduler, loggers),
            u"ShowOnKitchenFilterBox": ShowOnKitchenFilterBoxBuilder(product_repository, loggers),
            u"NoJitAddTagBox": NoJitAddTagBoxBuilder(product_repository, loggers),
            u"ItemPropertiesFilterBox": ItemPropertiesFilterBoxBuilder(loggers),
            u"AllServedItemsBox": AllServedItemsBoxBuilder(loggers),
            u"SendMomentTimerBox": SendMomentTimerBuilder(loggers),
            u"ConditionBox": ConditionBoxBuilder(loggers),
            u"CustomPropertyFilterBox": CustomPropertyFilterBoxBuilder(loggers),
            u"CustomParamPointerBox": CustomParamPointerBoxBuilder(product_repository, loggers),
            u"OrderRoundRobinBox": OrderRoundRobinBoxBuilder(order_changer, loggers),
            u"RemoveDeletedItemsBox": RemoveDeletedItemsBoxBuilder(loggers),
            u"FilterDeletedItemsBeforeFirstStateBox": FilterDeletedItemsBeforeFirstStateBoxBuilder(loggers),
            u"OrderSpreaderBox": OrderSpreaderBoxBuilder(loggers),
            u"OrderStampBox": OrderStampBoxBuilder(loggers),
            u"OrderMergerBox": OrderMergerBoxBuilder(loggers),
            u"RecalledOrderStampBox": RecalledOrderStampBoxBuilder(loggers),
            u"FilterOrderIfNotAllItemsHaveTagBox": FilterOrderIfNotAllItemsHaveTagBoxBuilder(loggers),
            u"AllItemsDoneBox": AllItemsDoneBoxBuilder(loggers),
            u"RemoveReprocessedOrdersBox": RemoveReprocessedOrdersBoxBuilder(loggers),
            u"ProductCodeFilterBox": ProductCodeFilterBoxBuilder(loggers),
            u"OrderProductCategoryFilterBox": OrderProductCategoryFilterBoxBuilder(loggers),
            u"PodTypeFilterBox": PodTypeFilterBoxBuilder(loggers),
            u"SaleTypeFilterBox": SaleTypeFilterBoxBuilder(loggers),
            u"SkipCourseBox": SkipCourseBoxBuilder(loggers),
            u"CollapseSameItemsBox": CollapseSameItemsBoxBuilder(loggers),
            u"ItemSequencerBox": ItemSequencerBoxBuilder(order_changer, loggers),
            u"StatisticBox": StatisticBoxBuilder(statistic_updater, loggers),
            u"AddTimeTagBox": AddTimeTagBoxBuilder(loggers),
            u"ProdStateFilterBox": ProdStateFilterBoxBuilder(product_repository, loggers),
            u"CustomPropertyValueFilterBox": CustomPropertyValueFilterBoxBuilder(loggers),
            u"ItemSorterBox": ItemSorterBoxBuilder(product_repository, loggers),
            u"SorterBox": SorterBoxBuilder(product_repository, loggers),
            u"DisplayTimeFilterBox": DisplayTimeFilterBoxBuilder(loggers),
            u"ProdStateTimerBox": ProdStateTimerBoxBuilder(loggers),
            u"DecreaseTimeTagBox": DecreaseTimeTagBoxBuilder(loggers),
            u"AllItemHasAtLeastOneTagFilterBox": AllItemHasAtLeastOneTagFilterBoxBuilder(loggers),
            u"FilterOrderIfAtLeastOneItemHasAtLeastOneTagFilterBox": FilterOrderIfAtLeastOneItemHasAtLeastOneTagFilterBoxBuilder(loggers),
            u"ComboTaggedWhenAllHasTagBox": ComboTaggedWhenAllHasTagBoxBuilder(loggers),
            u"TagItemsWhenProdStateBox": TagItemsWhenProdStateBoxBuilder(loggers),
            u"FilterOrderIfAllItemsHaveTagBox": FilterOrderIfAllItemsHaveTagBoxBuilder(loggers),
            u"FilterDefaultQuantityIngredientsBox": FilterDefaultQuantityIngredientsBoxBuilder(loggers),
            u"JitAddTagBox": JitAddTagBoxBuilder(product_repository, loggers),
            u"ReprocessOrdersBox": ReprocessOrdersBoxBuilder(publish_scheduler, loggers),
            u"JitSequencerItemFilterBox": JitSequencerItemFilterBoxBuilder(product_repository, loggers),
            u"SequencerTagItemBox": SequencerTagItemBoxBuilder(loggers),
            u"OrderSplitterBox": OrderSplitterBoxBuilder(order_changer, loggers),
            u"OrderPropertiesFilterBox": OrderPropertiesFilterBoxBuilder(order_changer, loggers)
        }

    def set_views(self, views):
        # type: (Dict[str, ProductionView]) -> None
        self.builders[u"ViewBox"] = ViewBoxBuilder(self.command_processor, self.order_changer, views, self.loggers)
        self.builders[u"StampItemsView"] = StampItemsViewBoxBuilder(self.order_changer, views, self.loggers)

    def build(self, config):
        # type: (BoxConfiguration) -> ProductionBox
        if config.type not in self.builders:
            raise Exception("Builder not found for box: {} - {}".format(config.name, config.type))

        return self.builders[config.type].build(config)



"""
PRNG management routines, thin wrappers.
"""

from OpenSSL._util import lib as _lib


def add(buffer, entropy):
    """
    Mix bytes from *string* into the PRNG state.

    The *entropy* argument is (the lower bound of) an estimate of how much
    randomness is contained in *string*, measured in bytes.

    For more information, see e.g. :rfc:`1750`.

    This function is only relevant if you are forking Python processes and
    need to reseed the CSPRNG after fork.

    :param buffer: Buffer with random data.
    :param entropy: The entropy (in bytes) measurement of the buffer.

    :return: :obj:`None`
    """
    if not isinstance(buffer, bytes):
        raise TypeError("buffer must be a byte string")

    if not isinstance(entropy, int):
        raise TypeError("entropy must be an integer")

    _lib.RAND_add(buffer, len(buffer), entropy)


def status():
    """
    Check whether the PRNG has been seeded with enough data.

    :return: 1 if the PRNG is seeded enough, 0 otherwise.
    """
    return _lib.RAND_status()



from old_helper import BaseRepository
from persistence import Connection


class PosCtrlRepository(BaseRepository):
    def __init__(self, mbcontext):
        super(PosCtrlRepository, self).__init__(mbcontext)

    def get_operators_qty(self, pos_id, business_period):
        # type: (int, unicode) -> int
        def inner_func(conn):
            # type: (Connection) -> int
            query = self._OperatorsQtyQuery
            query = query.format(pos_id, business_period)
            return int(conn.select(query).get_row(0).get_entry(0))

        return self.execute_with_connection(inner_func)

    _OperatorsQtyQuery = "SELECT count(*) from UserSession WHERE PosId='{0}' AND BusinessPeriod='{1}';"



class RequiredGroupOnSale(object):
    def __init__(self, obj):
        self.group = obj.get("group") if obj.get("group") else None
        self.quantity = int(obj.get("quantity")) if obj.get("quantity") else None



from typing import Any

from messageprocessorutil import UuidMessageProcessor

from dailygoals_services.interactor.update_total_sold import UpdateTotalSoldTableInteractor


class UpdateTotalSoldTableProcessor(UuidMessageProcessor):
    def __init__(self, interactor):
        # type: (UpdateTotalSoldTableInteractor) -> None

        super(UpdateTotalSoldTableProcessor, self).__init__()
        self.interactor = interactor

    def parse_data(self, data, event_type):
        # type: (Any, str) -> Any

        return data

    def call_business(self, params):
        # type: (Any) -> None

        self.interactor.update(params)

    def format_response(self, message_bus, message, event, request, response):
        return

    def format_exception(self, message_bus, message, event, request, exception):
        return

    def format_parse_exception(self, message_bus, message, event, data, exception):
        return



# -*- coding: utf-8 -*-
from model import StartPaymentData
from paymentrouter._PaymentRouter import PaymentError

from ._CommonProcessor import _CommonProcessor


class StartPaymentProcessor(_CommonProcessor):
    def parse_data(self, data, event_type):
        return StartPaymentData(*data.split(';'))

    def call_business(self, obj):
        # type: (StartPaymentData) -> str
        try:
            processor = self.payment_router.get_payment_processor(obj.electronic_type)
            add_payment_response = processor.add_payment(obj)
            return add_payment_response.to_xml()
        except PaymentError as ex:
            self.payment_router._logger.exception("exception: {}".format(ex.message))



from messagebus import TokenCreator, TokenPriority


class BusTokens(object):

    # RemoteOrder
    TK_REMOTE_ORDER_GET_RUPTURED_ITEMS = TokenCreator.create_token(TokenPriority.low, "37", "213")
    TK_REMOTE_ORDER_GET_MENU_DATA = TokenCreator.create_token(TokenPriority.low, "37", "226")

    # FiscalWrapper
    TK_FISCALWRAPPER_PROCESS_REQUEST = TokenCreator.create_token(TokenPriority.high, "31", "1")
    TK_FISCALWRAPPER_SEFAZ_CONNECTIVITY = TokenCreator.create_token(TokenPriority.high, "31", "4")
    TK_FISCALWRAPPER_SAT_OPERATIONAL_STATUS_REQUEST = TokenCreator.create_token(TokenPriority.high, "31", "5")
    TK_FISCALWRAPPER_GET_NF_TYPE = TokenCreator.create_token(TokenPriority.low, "31", "7")

    # EletronicPayment
    TK_ELECTRONIC_PAYMENT_ADD_PAYMENT = TokenCreator.create_token(TokenPriority.low, "0E", "1")
    TK_ELECTRONIC_PAYMENT_CANCEL_PAYMENT = TokenCreator.create_token(TokenPriority.low, "0E", "2")
    TK_ELECTRONIC_PAYMENT_SEND_RESPONSE = TokenCreator.create_token(TokenPriority.low, "0E", "5")

    # Coupons
    TK_COUPON_GET = TokenCreator.create_token(TokenPriority.low, "46", "2")
    TK_COUPON_GET_FREE_COUPON_DATA = TokenCreator.create_token(TokenPriority.low, "46", "B")
    TK_COUPON_APPLY = TokenCreator.create_token(TokenPriority.low, "113", "1")
    TK_COUPON_REMOVE = TokenCreator.create_token(TokenPriority.low, "112", "2")

    # PosUpdater
    TK_POS_UPDATER_GET_VERSIONS = TokenCreator.create_token(TokenPriority.low, "95", "6")

    # I18N
    TK_I18N_GETLANGS = TokenCreator.create_token(TokenPriority.low, "7", "4")
    TK_I18N_GETTABLEJSON = TokenCreator.create_token(TokenPriority.low, "7", "5")

    # Static Config
    TK_POS_EXECUTEACTION = TokenCreator.create_token(TokenPriority.high, "5", "3")



# -*- coding: utf-8 -*-

import base64
import os
import time

import re

from modelcommon.model import SitefResponse
from typing import (
    Any,
    Union
)

from model import (
    CancelPaymentOrderData,
    ConfirmFinishPaymentData,
    FinishPaymentData,
    PaymentResponseMessage,
    PaymentResponseData,
    TestConnectionData
)
from paymentprocessor import PaymentProcessor
from _CapptaCallback import CapptaCallback

cappta_client = None


class CapptaProcessor(PaymentProcessor):
    def __init__(self, message_bus, dialog_manager, config, logger):
        super(CapptaProcessor, self).__init__(message_bus, dialog_manager, config, logger)
        self.env = config.find_value("Environment")
        self.cnpj = config.find_value("CnpjLoja")
        self.pos_id = config.find_value("PosId")
        self.auth_key = config.find_value("ChaveAutenticacao")
        self.callback = CapptaCallback(self.dialog_manager)
        self.cappta_client = self._get_cappta_client(config)

    def connection_status(
        self,
        test_connection_data    # type: TestConnectionData
    ):
        # type: (...) -> bool

        return True

    def add_payment(self, start_payment_data):
        from Cappta.Gp.Api.Com.Model import DetalhesCredito, IMensagem, IRespostaTransacaoPendente, \
            IRequisicaoParametro, IRespostaOperacaoAprovada, IRespostaOperacaoRecusada

        try:
            pos_id = self.pos_id if self.env == 1 else 2
            ret = self.cappta_client.AutenticarPdv(self.cnpj, pos_id, self.auth_key)
            if ret == 0:
                self.logger.debug("POS %s - Autenticação com sucesso." % (str(pos_id)))
                self.cappta_client.IniciarMultiCartoes(9)
                tender_type = int(start_payment_data.tender_type)

                is_credit = tender_type == 1
                is_debit = tender_type == 2
                if is_credit:
                    details = DetalhesCredito()
                    ret = self.cappta_client.PagamentoCredito(float(start_payment_data.amount), details)
                elif is_debit:
                    ret = self.cappta_client.PagamentoDebito(float(start_payment_data.amount))

                if ret != 0:
                    self.callback.callback(str(self.pos_id), ret, "")
                    return

                tef_loop_variable = self.cappta_client.IterarOperacaoTef()
                while True:
                    if isinstance(tef_loop_variable, IMensagem):
                        message = IMensagem(tef_loop_variable)
                        msg = message.Descricao.encode('utf-8')
                        self.callback.callback(str(self.pos_id), msg, "MENSAGEM")
                        time.sleep(0.5)

                    elif isinstance(tef_loop_variable, IRespostaTransacaoPendente):
                        pending_transactions = IRespostaTransacaoPendente(tef_loop_variable)
                        msg = pending_transactions.Mensagem.encode('utf-8')
                        callback_response = self.callback.callback(str(self.pos_id), msg, "REQUISITA_ACAO")
                        if callback_response == "CANCELA":
                            callback_response = "2"
                        ret = self.cappta_client.EnviarParametro(callback_response, 1)

                    elif isinstance(tef_loop_variable, IRequisicaoParametro):
                        req_parameter = IRequisicaoParametro(tef_loop_variable)
                        msg = req_parameter.Mensagem.encode('utf-8')
                        callback_response = self.callback.callback(str(self.pos_id), msg, "REQUISITA_ACAO")

                        if callback_response == "CANCELA":
                            ret = self.cappta_client.EnviarParametro("", 2)
                        else:
                            ret = self.cappta_client.EnviarParametro(callback_response, 1)

                    if isinstance(tef_loop_variable, IRespostaOperacaoAprovada) or isinstance(tef_loop_variable,
                                                                                              IRespostaOperacaoRecusada):
                        break

                    tef_loop_variable = self.cappta_client.IterarOperacaoTef()

                if isinstance(tef_loop_variable, IRespostaOperacaoAprovada):
                    approved_payment = IRespostaOperacaoAprovada(tef_loop_variable)
                    auth_code = str(approved_payment.CodigoAutorizacaoAdquirente)
                    nome_instituicao = str(approved_payment.NomeBandeiraCartao)
                    card_bin = str(approved_payment.NumeroCartaoCliente)[:6]
                    rede_autorizadora = ""
                    transaction_processor = ""
                    bandeira = str(approved_payment.CodigoBandeiraCartao)

                    customer_slip = approved_payment.CupomLojista
                    b64_receipt_merchant = base64.b64encode(str(customer_slip).replace("\"", ""))
                    client_slip = approved_payment.CupomCliente
                    b64_receipt_customer = base64.b64encode(str(client_slip).replace("\"", ""))

                    response = PaymentResponseData(
                        media=nome_instituicao,
                        result=ret,
                        card_number=card_bin,
                        auth_code=auth_code,
                        approved_amount=start_payment_data.amount,
                        id_auth=rede_autorizadora,
                        cnpj_auth="",
                        bandeira=bandeira,
                        receipt_merchant=b64_receipt_merchant,
                        receipt_customer=b64_receipt_customer,
                        expiration_date="",
                        nsu="",
                        transaction_processor=transaction_processor,
                        last_digits="",
                        sat_acquirer="",
                        nfce_brand="",
                        original_nfce_brand="",
                        nsu_host="",
                        electronic_type=start_payment_data.electronic_type,
                        transaction_identifier=""
                    )

                    msg = "POS %s - PaymentApproved. Order %s" % (str(self.pos_id), str(start_payment_data.order_id))
                    self.logger.debug(msg)
                    return response

                elif isinstance(tef_loop_variable, IRespostaOperacaoRecusada):
                    operation = IRespostaOperacaoRecusada(tef_loop_variable)
                    response = operation.Motivo.encode('utf-8')
                    self.logger.debug("POS %s - Operação recusada. Order: %s. Motivo: %s" % (
                    str(self.pos_id), str(start_payment_data.order_id), response))
                    response = "<EftXml Result=\"%s\" />" % response
                    return response
            else:
                message = "Falha ao autenticar POS %s - Código de erro %s" % (str(self.pos_id), str(ret))
                self.logger.error(message)
                return PaymentResponseMessage(message)
        finally:
            self.dialog_manager.close_dialogs(start_payment_data.pos_id)

    def cancel_payment_order(
        self,
        cancel_payment_order_data     # type: CancelPaymentOrderData
    ):
        # type: (...) -> Any[Union[SitefResponse,str]]

        # TODO: Check if need to implement
        pass

    def cancel_payment(self):
        return

    def finish_payment(
        self,
        finish_payment_data     # type: FinishPaymentData
    ):
        # type: (...) -> bool

        try:
            if finish_payment_data.status == 1:
                self.cappta_client.ConfirmarPagamentos()
            else:
                self.cappta_client.DesfazerPagamentos()
            return True
        except Exception as _:
            return False

    def confirm_finish_payment(
        self,
        confirm_finish_payment_data     # type: ConfirmFinishPaymentData
    ):
        # type: (...) -> None

        pass

    def get_name(self):
        return "Cappta"

    def stop(self):
        return

    def _get_cappta_client(self, config):
        import clr  # Linux (POS Mobile) does not have an implemented library to clr

        global cappta_client
        if cappta_client is not None:
            return cappta_client

        dll_path = self._get_dll_path(config.find_value("DLLPath"))
        old_dir = os.getcwd()
        os.chdir(dll_path)
        clr.AddReference(os.path.join(os.getcwd(), "Cappta.Gp.Api.Com.dll"))

        from Cappta.Gp.Api.Com import ClienteCappta

        cappta_client = ClienteCappta()
        os.chdir(old_dir)

        return cappta_client

    @staticmethod
    def _get_dll_path(dll_path):
        for var in re.findall("{[a-zA-Z0-9]*}", dll_path):
            dll_path = dll_path.replace(var, os.environ.get(var.replace('{', '').replace('}', ''), ""))
        return os.path.normpath(dll_path)



#
#   ASN.1 subtype constraints classes.
#
#   Constraints are relatively rare, but every ASN1 object
#   is doing checks all the time for whether they have any
#   constraints and whether they are applicable to the object.
#
#   What we're going to do is define objects/functions that
#   can be called unconditionally if they are present, and that
#   are simply not present if there are no constraints.
#
#   Original concept and code by Mike C. Fletcher.
#
import sys
from pyasn1.type import error

class AbstractConstraint:
    """Abstract base-class for constraint objects

       Constraints should be stored in a simple sequence in the
       namespace of their client Asn1Item sub-classes.
    """
    def __init__(self, *values):
        self._valueMap = {}
        self._setValues(values)
        self.__hashedValues = None
    def __call__(self, value, idx=None):
        try:
            self._testValue(value, idx)
        except error.ValueConstraintError:
            raise error.ValueConstraintError(
               '%s failed at: \"%s\"' % (self, sys.exc_info()[1])
            )
    def __repr__(self):
        return '%s(%s)' % (
            self.__class__.__name__,
            ', '.join([repr(x) for x in self._values])
        )
    def __eq__(self, other):
        return self is other and True or self._values == other
    def __ne__(self, other): return self._values != other
    def __lt__(self, other): return self._values < other
    def __le__(self, other): return self._values <= other
    def __gt__(self, other): return self._values > other
    def __ge__(self, other): return self._values >= other
    if sys.version_info[0] <= 2:
        def __nonzero__(self): return bool(self._values)
    else:
        def __bool__(self): return bool(self._values)

    def __hash__(self):
        if self.__hashedValues is None:
            self.__hashedValues = hash((self.__class__.__name__, self._values))
        return self.__hashedValues

    def _setValues(self, values): self._values = values
    def _testValue(self, value, idx):
        raise error.ValueConstraintError(value)

    # Constraints derivation logic
    def getValueMap(self): return self._valueMap
    def isSuperTypeOf(self, otherConstraint):
        return self in otherConstraint.getValueMap() or \
               otherConstraint is self or otherConstraint == self
    def isSubTypeOf(self, otherConstraint):
        return otherConstraint in self._valueMap or \
               otherConstraint is self or otherConstraint == self

class SingleValueConstraint(AbstractConstraint):
    """Value must be part of defined values constraint"""
    def _testValue(self, value, idx):
        # XXX index vals for performance?
        if value not in self._values:
            raise error.ValueConstraintError(value)

class ContainedSubtypeConstraint(AbstractConstraint):
    """Value must satisfy all of defined set of constraints"""
    def _testValue(self, value, idx):
        for c in self._values:
            c(value, idx)

class ValueRangeConstraint(AbstractConstraint):
    """Value must be within start and stop values (inclusive)"""
    def _testValue(self, value, idx):
        if value < self.start or value > self.stop:
            raise error.ValueConstraintError(value)

    def _setValues(self, values):
        if len(values) != 2:
            raise error.PyAsn1Error(
                '%s: bad constraint values' % (self.__class__.__name__,)
                )
        self.start, self.stop = values
        if self.start > self.stop:
            raise error.PyAsn1Error(
                '%s: screwed constraint values (start > stop): %s > %s' % (
                    self.__class__.__name__,
                    self.start, self.stop
                )
            )
        AbstractConstraint._setValues(self, values)
        
class ValueSizeConstraint(ValueRangeConstraint):
    """len(value) must be within start and stop values (inclusive)"""
    def _testValue(self, value, idx):
        l = len(value)
        if l < self.start or l > self.stop:
            raise error.ValueConstraintError(value)

class PermittedAlphabetConstraint(SingleValueConstraint):
    def _setValues(self, values):
        self._values = ()
        for v in values:
            self._values = self._values + tuple(v)

    def _testValue(self, value, idx):
        for v in value:
            if v not in self._values:
                raise error.ValueConstraintError(value)

# This is a bit kludgy, meaning two op modes within a single constraing
class InnerTypeConstraint(AbstractConstraint):
    """Value must satisfy type and presense constraints"""
    def _testValue(self, value, idx):
        if self.__singleTypeConstraint:
            self.__singleTypeConstraint(value)
        elif self.__multipleTypeConstraint:
            if idx not in self.__multipleTypeConstraint:
                raise error.ValueConstraintError(value)
            constraint, status = self.__multipleTypeConstraint[idx]
            if status == 'ABSENT':   # XXX presense is not checked!
                raise error.ValueConstraintError(value)
            constraint(value)

    def _setValues(self, values):
        self.__multipleTypeConstraint = {}
        self.__singleTypeConstraint = None
        for v in values:
            if isinstance(v, tuple):
                self.__multipleTypeConstraint[v[0]] = v[1], v[2]
            else:
                self.__singleTypeConstraint = v
        AbstractConstraint._setValues(self, values)

# Boolean ops on constraints 

class ConstraintsExclusion(AbstractConstraint):
    """Value must not fit the single constraint"""
    def _testValue(self, value, idx):
        try:
            self._values[0](value, idx)
        except error.ValueConstraintError:
            return
        else:
            raise error.ValueConstraintError(value)

    def _setValues(self, values):
        if len(values) != 1:
            raise error.PyAsn1Error('Single constraint expected')
        AbstractConstraint._setValues(self, values)

class AbstractConstraintSet(AbstractConstraint):
    """Value must not satisfy the single constraint"""
    def __getitem__(self, idx): return self._values[idx]

    def __add__(self, value): return self.__class__(self, value)
    def __radd__(self, value): return self.__class__(self, value)

    def __len__(self): return len(self._values)

    # Constraints inclusion in sets
    
    def _setValues(self, values):
        self._values = values
        for v in values:
            self._valueMap[v] = 1
            self._valueMap.update(v.getValueMap())

class ConstraintsIntersection(AbstractConstraintSet):
    """Value must satisfy all constraints"""
    def _testValue(self, value, idx):
        for v in self._values:
            v(value, idx)

class ConstraintsUnion(AbstractConstraintSet):
    """Value must satisfy at least one constraint"""
    def _testValue(self, value, idx):
        for v in self._values:
            try:
                v(value, idx)
            except error.ValueConstraintError:
                pass
            else:
                return
        raise error.ValueConstraintError(
            'all of %s failed for \"%s\"' % (self._values, value)
            )

# XXX
# add tests for type check



from ._TableOrderSaleLine import TableOrderSaleLine  # noqa
from ._OrderStateEnum import OrderStateEnum  # noqa
from typing import List  # noqa


class TableOrder(object):

    def __init__(self, order_id, order_state, total_amount, total_after_discount, total_tender, discount_amount,
                 sales_line, split_key, tip):
        # type: (str, OrderStateEnum, float, float, float, float, List[TableOrderSaleLine], str, float) -> None
        self.order_id = order_id
        self.order_state = order_state
        self.total_amount = total_amount
        self.total_after_discount = total_after_discount
        self.total_tender = total_tender
        self.discount_amount = discount_amount
        self.sales_line = sales_line
        self.split_key = split_key
        self.tip = tip



class SitefActions(object):
    SHOW_OPERATOR_MESSAGE = 1
    SHOW_OPERATOR_AND_CUSTOMER_MESSAGE = 3
    MENU_TITLE = 4
    CLEAN_OPERATOR_MESSAGE = 11
    CLEAN_CUSTOMER_MESSAGE = 12
    CLEAN_OPERATOR_AND_CUSTOMER_MESSAGE = 13
    CLEAN_MENU_TITLE = 14
    CANCEL_OPERATION = 20
    MENU_CONTENT = 21
    SHOW_MESSAGE_WITH_CONFIRMATION = 22
    SHOW_MESSAGE_WITH_CANCEL_BUTTON = 23
    ASK_PIN_CODE = 30
    INSERT_SECURITY_CODE = 30
    CLOSE_QRCODE = 51



from ._AddBenefitToApplyInteractor import AddBenefitToApplyInteractor
from ._RemoveBenefitInteractor import RemoveBenefitInteractor
from ._GetBenefitsInteractor import GetBenefitsInteractor
from ._OnOrderReopenInteractor import OnOrderReopenInteractor
from ._OnSaleItemInteractor import OnSaleItemInteractor
from ._OnOrderTotalInteractor import OnOrderTotalInteractor
from ._OnTableOpenInteractor import OnTableOpenInteractor
from ._OnTabOpenInteractor import OnTabOpenInteractor



from mebuhi.event import EventPublisher
from messagebus import NakMessage
from messagehandler import MessageHandler

class MebuhiMessageHandler(MessageHandler):
    def __init__(self, event_publisher):
        # type: (EventPublisher) -> None
        self.event_publisher = event_publisher

    def handle_event(self, message_bus, event):
        self.event_publisher.new_event(event)

    def handle_message(self, message_bus, message):
        message_bus.reply_message(message, NakMessage())



import sys
from lib2to3 import refactor

# The original set of these fixes comes from lib3to2 (https://bitbucket.org/amentajo/lib3to2):
fix_names = set([
                 'libpasteurize.fixes.fix_add_all__future__imports',  # from __future__ import absolute_import etc. on separate lines
                 'libpasteurize.fixes.fix_add_future_standard_library_import',  # we force adding this import for now, even if it doesn't seem necessary to the fix_future_standard_library fixer, for ease of testing
                 # 'libfuturize.fixes.fix_order___future__imports',  # consolidates to a single line to simplify testing -- UNFINISHED
                 'libpasteurize.fixes.fix_future_builtins',   # adds "from future.builtins import *"
                 'libfuturize.fixes.fix_future_standard_library', # adds "from future import standard_library"

                 'libpasteurize.fixes.fix_annotations',
                 # 'libpasteurize.fixes.fix_bitlength',  # ints have this in Py2.7
                 # 'libpasteurize.fixes.fix_bool',    # need a decorator or Mixin
                 # 'libpasteurize.fixes.fix_bytes',   # leave bytes as bytes
                 # 'libpasteurize.fixes.fix_classdecorator',  # available in
                 # Py2.6+
                 # 'libpasteurize.fixes.fix_collections', hmmm ...
                 # 'libpasteurize.fixes.fix_dctsetcomp',  # avail in Py27
                 'libpasteurize.fixes.fix_division',   # yes
                 # 'libpasteurize.fixes.fix_except',   # avail in Py2.6+
                 # 'libpasteurize.fixes.fix_features',  # ?
                 'libpasteurize.fixes.fix_fullargspec',
                 # 'libpasteurize.fixes.fix_funcattrs',
                 'libpasteurize.fixes.fix_getcwd',
                 'libpasteurize.fixes.fix_imports',   # adds "from future import standard_library"
                 'libpasteurize.fixes.fix_imports2',
                 # 'libpasteurize.fixes.fix_input',
                 # 'libpasteurize.fixes.fix_int',
                 # 'libpasteurize.fixes.fix_intern',
                 # 'libpasteurize.fixes.fix_itertools',
                 'libpasteurize.fixes.fix_kwargs',   # yes, we want this
                 # 'libpasteurize.fixes.fix_memoryview',
                 # 'libpasteurize.fixes.fix_metaclass',  # write a custom handler for
                 # this
                 # 'libpasteurize.fixes.fix_methodattrs',  # __func__ and __self__ seem to be defined on Py2.7 already
                 'libpasteurize.fixes.fix_newstyle',   # yes, we want this: explicit inheritance from object. Without new-style classes in Py2, super() will break etc.
                 # 'libpasteurize.fixes.fix_next',   # use a decorator for this
                 # 'libpasteurize.fixes.fix_numliterals',   # prob not
                 # 'libpasteurize.fixes.fix_open',   # huh?
                 # 'libpasteurize.fixes.fix_print',  # no way
                 'libpasteurize.fixes.fix_printfunction',  # adds __future__ import print_function
                 # 'libpasteurize.fixes.fix_raise_',   # TODO: get this working!

                 # 'libpasteurize.fixes.fix_range',  # nope
                 # 'libpasteurize.fixes.fix_reduce',
                 # 'libpasteurize.fixes.fix_setliteral',
                 # 'libpasteurize.fixes.fix_str',
                 # 'libpasteurize.fixes.fix_super',  # maybe, if our magic super() isn't robust enough
                 'libpasteurize.fixes.fix_throw',   # yes, if Py3 supports it
                 # 'libpasteurize.fixes.fix_unittest',
                 'libpasteurize.fixes.fix_unpacking',  # yes, this is useful
                 # 'libpasteurize.fixes.fix_with'      # way out of date
                ])




# -*- coding: utf-8 -*-

from datetime import datetime
from json import JSONEncoder

from chatservice.model import Message


class ChatModelJsonEncoder(JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Message):
            ret_dict = {}
            if obj.id is not None:
                ret_dict["id"] = obj.id
            if obj.message_from is not None:
                ret_dict["from"] = obj.message_from
            if obj.created_time is not None:
                ret_dict["createdTime"] = obj.created_time
            if obj.received_time is not None:
                ret_dict["receivedTime"] = obj.received_time
            if obj.text is not None:
                ret_dict["text"] = obj.text
            if obj.server_id is not None:
                ret_dict["serverId"] = obj.server_id

            return ret_dict

        if isinstance(obj, datetime):
            return obj.strftime("%Y-%m-%dT%H:%M:%S.%fZ")

        return super(ChatModelJsonEncoder, self).default(obj)



from systools import sys_log_error
from msgbus import TK_POS_GETPOSLIST, TK_SYS_ACK

pos_list = None


def get_pos_list(mb_context):
    global pos_list

    if pos_list is None:
        msg = mb_context.MB_EasySendMessage("PosController", TK_POS_GETPOSLIST)
        if msg.token != TK_SYS_ACK:
            sys_log_error("Could not retrieve PosList")
            raise Exception("Could not retrieve PosList")

        pos_list = sorted(map(int, msg.data.split("\0")))

    return pos_list



from logging import Logger
from typing import Any

from model.error_type import ExternalServicesErrorsType
from interactor.route.payment.insert.dto import (
    OrderInsertPaymentRequest,
    OrderInsertPaymentResponse
)
from model.config import Config
from model.exception import ExternalServiceErrors
from model.repository import PosRepositoryModel
from validation_util import \
    check_if_business_day_is_opened, \
    check_if_operator_is_logged, \
    check_if_order_is_totaled, \
    check_if_payment_type_is_valid, \
    check_if_payment_amount_is_valid, \
    validate_api_key


class OrderInsertPaymentInteractor(object):
    def __init__(self, logger, config, pos_repository, fiscal_service):
        # type: (Logger, Config, PosRepositoryModel, Any) -> None

        self.logger = logger
        self.config = config
        self.pos_repository = pos_repository
        self.fiscal_service = fiscal_service

    def execute(self, request):
        # type: (OrderInsertPaymentRequest) -> OrderInsertPaymentResponse

        pos_id = validate_api_key(request, self.config.api_key_mapping)
        model = self.pos_repository.get_model(pos_id)
        check_if_business_day_is_opened(self.pos_repository, model)
        check_if_operator_is_logged(self.pos_repository, model)
        check_if_order_is_totaled(self.pos_repository, model)
        check_if_payment_type_is_valid(self.config.allowed_payment_methods, request.payment_id)
        check_if_payment_amount_is_valid(request.amount, model)

        pos_ot = self.pos_repository.get_pos_ot(model)

        try:
            self.fiscal_service.process_payment(model, request)
        except Exception as _:
            self.logger.exception("Error performing payment")
            raise ExternalServiceErrors(ExternalServicesErrorsType.InsertPaymentUnavailable)

        tender = self.pos_repository.do_tender(pos_id, request.payment_id, pos_ot, request.amount)
        response = OrderInsertPaymentResponse(tender)

        return response



import time
from logging import Logger
from threading import Thread

from messagebus import MessageBus, TokenCreator, TokenPriority, DataType, Message, DefaultToken, MessageBusException
from mw_helper import ensure_list
from production.model import ProductionOrder
from production.view import ProductionView
from production.view._OrderXml import OrderXml

PERIPHERALS_GROUP = "6"
REPORT_GROUP = "D"

TK_PRN_PRINT = TokenCreator.create_token(TokenPriority.high, PERIPHERALS_GROUP, "101")
TK_REPORT_GENERATE = TokenCreator.create_token(TokenPriority.low, REPORT_GROUP, "1")

MAX_RETRY = 10


class PrinterView(ProductionView):
    """ class PrinterView(ProductionView)
    Production view used that formats a report (or slip) and prints it.
    """

    def __init__(self, name, message_bus, report, printer, picklist_name, timeout, tags_on_print=None, logger=None):
        # type: (str, MessageBus, str, str, str, int, str, Logger) -> None

        super(PrinterView, self).__init__(name, logger)
        self.message_bus = message_bus
        self.report = report
        self.printer = printer
        self.picklist_name = picklist_name
        self.timeout = timeout
        self.tags_on_print = ensure_list(tags_on_print)
        self.logger = logger
        self.order_xml = OrderXml()

    def handle_order(self, order, synch=False):
        # type: (ProductionOrder, bool) -> Thread

        t = Thread(target=self.generate_and_print_report, args=[order])
        t.daemon = True
        t.start()
        return t

    def generate_and_print_report(self, order):
        xml = self.order_xml.to_xml(order)
        report = None
        for _ in range(1, 5):
            try:
                report = self._generate(xml)
                break
            except BaseException: # noqa
                self.exception("Exception trying to generate order id {}", order.order_id)
                time.sleep(1)

        if report is None or report == "":
            return

        for attempt in range(1, MAX_RETRY):
            if self._print(report, attempt):
                break
            time.sleep(2 * attempt)

    def get_view_tags(self):
        return self.tags_on_print

    def _print(self, text, attempt):
        try:
            self.debug("Sending to printer: {} - Attempt: {}", self.printer, attempt)

            if attempt == 1:
                self.debug("Printer: {} - \nPrinting:\n {}", self.printer, text)

            message = Message(TK_PRN_PRINT, DataType.string, text, self.timeout)
            reply = self.message_bus.send_message(self.printer, message)
            if reply.token != DefaultToken.TK_SYS_ACK:
                self.error("Invalid response from printer: {} / {} - {}", self.printer, reply.token, reply.data)
                return False
            else:
                self.debug("Successfully printed on pinter: {}".format(self.printer))
                return True
        except BaseException as _:
            if attempt >= MAX_RETRY - 1:
                self.exception("Printer unavailable - Failed to Print Report")
            else:
                self.debug("Error when trying to print on Printer {} in attempt {}".format(self.printer, attempt))
            return False

    def _generate(self, data):
        msg_params = "%s\0%s\0%s" % (self.report, data, self.picklist_name)
        message = Message(TK_REPORT_GENERATE, DataType.param, msg_params, self.timeout)
        reply = self.message_bus.send_message("ReportsGenerator", message)

        if reply.token == DefaultToken.TK_SYS_ACK:
            self.debug("Report successfully generated")
            return reply.data

        else:
            self.error("Invalid reply while generating report: {} - {}", reply.token, reply.data)

        return None



from model import PaymentResponse


class PaymentResponseMessage(PaymentResponse):
    def __init__(self, message):
        self.message = message

    def to_xml(self):
        return """<EftXml Result="{}"/>""".format(self.message)



# -*- coding: utf-8 -*-
from dialogmanager import DialogManager
from messagehandler import MessageHandlerBuilder
from messagepaymentprocessor import (
    CancelPaymentProcessor,
    CancelPaymentOrderProcessor,
    ConfirmFinishPaymentProcessor,
    FinishPaymentProcessor,
    GetElectronicTypesProcessor,
    TestConnectionProcessor,
    StartPaymentProcessor
)
from messageprocessor import DefaultMessageProcessorExecutorFactory, MessageProcessorMessageHandler
from messageprocessorutil import LoggingProcessorCallback
from messagetoken import MessageToken
from paymentrouter import PaymentRouter
from typing import Optional


class ElectronicPaymentMessageHandlerBuilder(MessageHandlerBuilder):
    def __init__(self, message_bus, config, logger):
        self.message_bus = message_bus
        self.config = config
        self.logger = logger

        self.message_handler = None
        self.dialog_manager = None  # type: Optional[DialogManager]
        self.payment_router = None  # type: Optional[PaymentRouter]

    def build_singletons(self):
        self.dialog_manager = DialogManager(self.message_bus, self.logger)
        self.payment_router = PaymentRouter(self.message_bus, self.dialog_manager, self.config, self.logger)

    def build_message_handler(self, _, __):
        event_processors = {}
        message_processors = {
            MessageToken.TK_ELECTRONIC_PAYMENT_ADD_PAYMENT: StartPaymentProcessor(self.payment_router),
            MessageToken.TK_ELECTRONIC_PAYMENT_CANCEL_PAYMENT: CancelPaymentProcessor(self.payment_router),
            MessageToken.TK_ELECTRONIC_PAYMENT_CANCEL_PAYMENT_ORDER: CancelPaymentOrderProcessor(self.payment_router),
            MessageToken.TK_ELECTRONIC_PAYMENT_FINISH_PAYMENT: FinishPaymentProcessor(self.payment_router),
            MessageToken.TK_ELECTRONIC_PAYMENT_CONFIRM_FINISH: ConfirmFinishPaymentProcessor(self.payment_router),
            MessageToken.TK_ELECTRONIC_PAYMENT_CONNECTIVITY_TEST: TestConnectionProcessor(self.payment_router),
            MessageToken.TK_ELECTRONIC_PAYMENT_GET_ELECTRONIC_TYPES: GetElectronicTypesProcessor(self.payment_router),
            MessageToken.TK_ELECTRONIC_PAYMENT_SEND_RESPONSE: self.dialog_manager
        }
        self.message_handler = MessageProcessorMessageHandler(
            event_processors,
            message_processors,
            DefaultMessageProcessorExecutorFactory([LoggingProcessorCallback(self.logger)]),
            None
        )
        return self.message_handler

    def destroy_message_handler(self, message_handler):
        pass  # no special handling necessary to destroy message handlers

    def destroy_singletons(self):
        self.payment_router.stop_processors()



from abc import ABCMeta, abstractmethod
from typing import Optional
from ._DefaultMessageProcessorExecutor import MessageProcessor
from ._ApiRequest import ApiRequest


class ApiRouter(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_message_processor(self, api_request):
        # type: (ApiRequest) -> Optional[MessageProcessor]
        raise NotImplementedError()



# -*- coding: utf-8 -*-
"""
    flask
    ~~~~~

    A microframework based on Werkzeug.  It's extensively documented
    and follows best practice patterns.

    :copyright: 2010 Pallets
    :license: BSD-3-Clause
"""
# utilities we import from Werkzeug and Jinja2 that are unused
# in the module but are exported as public interface.
from jinja2 import escape
from jinja2 import Markup
from werkzeug.exceptions import abort
from werkzeug.utils import redirect

from . import json
from ._compat import json_available
from .app import Flask
from .app import Request
from .app import Response
from .blueprints import Blueprint
from .config import Config
from .ctx import after_this_request
from .ctx import copy_current_request_context
from .ctx import has_app_context
from .ctx import has_request_context
from .globals import _app_ctx_stack
from .globals import _request_ctx_stack
from .globals import current_app
from .globals import g
from .globals import request
from .globals import session
from .helpers import flash
from .helpers import get_flashed_messages
from .helpers import get_template_attribute
from .helpers import make_response
from .helpers import safe_join
from .helpers import send_file
from .helpers import send_from_directory
from .helpers import stream_with_context
from .helpers import url_for
from .json import jsonify
from .signals import appcontext_popped
from .signals import appcontext_pushed
from .signals import appcontext_tearing_down
from .signals import before_render_template
from .signals import got_request_exception
from .signals import message_flashed
from .signals import request_finished
from .signals import request_started
from .signals import request_tearing_down
from .signals import signals_available
from .signals import template_rendered
from .templating import render_template
from .templating import render_template_string

__version__ = "1.1.1"



# -*- coding: utf-8 -*-
from threading import Lock

import utils
import actions.util
from actions.login.get_user_info import get_user_info
from model.customexception import GettingUserInfoException, CanceledDialogException

from sysactions import (
    action,
    get_model,
    show_messagebox,
    StopAction,
    AuthenticationFailed
)

from login_user import sign_in_quick_user
from signin_user import sign_in_table_user
from actions.pos import pos_must_be_blocked
from typing import Optional

current_pos_login_users = {}
login_lock = Lock()
mbcontext = None

@action
def common_login(
        pos_id,         # type: str
        user_id=None,   # type: Optional[str]
        password=None   # type: Optional[str]
):
    # type: (...) -> bool

    global current_pos_login_users

    model = get_model(pos_id)
    try:
        user_info = get_user_info(pos_id, user_id, password, model)
        utils.validate_user_level(pos_id, user_info)
    except CanceledDialogException as _:
        raise StopAction()
    except (AuthenticationFailed, GettingUserInfoException) as _:
        show_messagebox(pos_id, "$INVALID_USERNAME_OR_PASSWORD", "$INFORMATION")
        raise StopAction()

    user_id = user_info.get("UserId")
    with login_lock:
        logged_pos_id = current_pos_login_users[user_id] if user_id in current_pos_login_users else None

    if logged_pos_id is not None:
        show_messagebox(pos_id, message="$USER_IS_LOGIN_IN_POS|{}".format(logged_pos_id), icon="error")
        raise StopAction()

    try:
        with login_lock:
            current_pos_login_users[user_id] = pos_id

        _login_validations(pos_id, user_id)

        working_mode = model.find("WorkingMode").get("usrCtrlType")
        if working_mode == "QSR":
            return sign_in_quick_user(pos_id, user_info)
        else:
            return sign_in_table_user(pos_id, user_info)
    finally:
        with login_lock:
            current_pos_login_users[user_id] = None


def _login_validations(pos_id, user_id):
    if pos_must_be_blocked(pos_id):
        show_messagebox(pos_id, message="$POS_BLOCKED", icon="error")
        raise StopAction()

    if not utils.check_current_day(pos_id):
        raise StopAction()

    actions.util.set_context(mbcontext)
    if actions.util.is_user_already_logged(pos_id, user_id):
        raise StopAction()



from unicodedata import normalize

from old_helper import BaseRepository
from typing import List, Tuple
from persistence import Connection
from helper import remove_accents
from ..model import Order

BAND_DESCRIPTION_CACHE = None


class FiscalRepository(BaseRepository):
    def __init__(self, mbcontext):
        super(FiscalRepository, self).__init__(mbcontext)

    def get_card_brands(self, paid_orders):
        # type: (List[Order]) -> List[Tuple[unicode, int, float, int]]
        def inner_func(conn):
            # type: (Connection) -> List[Tuple[unicode, int, float, int]]
            order_list = map(lambda order: order.order_id, paid_orders)
            order_list = ','.join(map(str, order_list))
            query = self._CardBrandsQuery
            query = query.format(order_list)
            return [((x.get_entry(0)).decode('utf-8'), int(x.get_entry(1)), float(x.get_entry(2)), int(x.get_entry(3))) for x in conn.select(query)]

        return self.execute_with_connection(inner_func, service_name="FiscalPersistence")

    def get_order_id_by_nf_number(self, orders):
        # type: (List[Order]) -> List[Tuple[int, int]]
        def inner_func(conn):
            # type: (Connection) -> List[Tuple[int, int]]
            order_list = map(lambda order: "'{0}'".format(order.nf_number), orders)
            order_list = ','.join(order_list)
            query = self._NumberNfQuery
            query = query.format(order_list)
            nfs_number = {}
            for x in conn.select(query):
                nfs_number[x.get_entry("NumeroNota")] = int(x.get_entry("OrderId"))
            return nfs_number

        return self.execute_with_connection(inner_func, None, "FiscalPersistence")

    _CardBrandsQuery = """SELECT
                          CASE WHEN bc.Descricao IS NOT NULL THEN bc.Descricao ELSE 'OUTROS' END AS Brand, pd.Type as Type, SUM(pd.Amount) as Amount, COUNT(pd.OrderId)
                          FROM fiscal.PaymentData pd
                          LEFT JOIN fiscal.BandeiraCartao bc on pd.Bandeira = bc.Bandeira
                          WHERE pd.Type IN (1,2) AND pd.OrderId IN ({0})
                          GROUP BY bc.Descricao, pd.Type
                          ORDER BY Type ASC, Amount DESC"""

    _NumberNfQuery = """select fd.OrderId, fd.NumeroNota
                                  from fiscal.FiscalData fd
                                  where fd.NumeroNota in ({0})
                                  order by fd.OrderId asc;"""

    def get_brands_description(self):
        global BAND_DESCRIPTION_CACHE

        def inner_func(conn):
            query = "select bandeira, descricao from bandeiracartao"
            card_brands = {}
            for row in conn.select(query):
                card_brands[int(row.get_entry(0))] = remove_accents(row.get_entry(1))
            return card_brands

        if not BAND_DESCRIPTION_CACHE:
            BAND_DESCRIPTION_CACHE = self.execute_with_connection(inner_func, service_name="FiscalPersistence")
        return BAND_DESCRIPTION_CACHE



######################## BEGIN LICENSE BLOCK ########################
# The Original Code is mozilla.org code.
#
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 1998
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Mark Pilgrim - port to Python
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301  USA
######################### END LICENSE BLOCK #########################

from .enums import MachineState

# BIG5

BIG5_CLS = (
    1,1,1,1,1,1,1,1,  # 00 - 07    #allow 0x00 as legal value
    1,1,1,1,1,1,0,0,  # 08 - 0f
    1,1,1,1,1,1,1,1,  # 10 - 17
    1,1,1,0,1,1,1,1,  # 18 - 1f
    1,1,1,1,1,1,1,1,  # 20 - 27
    1,1,1,1,1,1,1,1,  # 28 - 2f
    1,1,1,1,1,1,1,1,  # 30 - 37
    1,1,1,1,1,1,1,1,  # 38 - 3f
    2,2,2,2,2,2,2,2,  # 40 - 47
    2,2,2,2,2,2,2,2,  # 48 - 4f
    2,2,2,2,2,2,2,2,  # 50 - 57
    2,2,2,2,2,2,2,2,  # 58 - 5f
    2,2,2,2,2,2,2,2,  # 60 - 67
    2,2,2,2,2,2,2,2,  # 68 - 6f
    2,2,2,2,2,2,2,2,  # 70 - 77
    2,2,2,2,2,2,2,1,  # 78 - 7f
    4,4,4,4,4,4,4,4,  # 80 - 87
    4,4,4,4,4,4,4,4,  # 88 - 8f
    4,4,4,4,4,4,4,4,  # 90 - 97
    4,4,4,4,4,4,4,4,  # 98 - 9f
    4,3,3,3,3,3,3,3,  # a0 - a7
    3,3,3,3,3,3,3,3,  # a8 - af
    3,3,3,3,3,3,3,3,  # b0 - b7
    3,3,3,3,3,3,3,3,  # b8 - bf
    3,3,3,3,3,3,3,3,  # c0 - c7
    3,3,3,3,3,3,3,3,  # c8 - cf
    3,3,3,3,3,3,3,3,  # d0 - d7
    3,3,3,3,3,3,3,3,  # d8 - df
    3,3,3,3,3,3,3,3,  # e0 - e7
    3,3,3,3,3,3,3,3,  # e8 - ef
    3,3,3,3,3,3,3,3,  # f0 - f7
    3,3,3,3,3,3,3,0  # f8 - ff
)

BIG5_ST = (
    MachineState.ERROR,MachineState.START,MachineState.START,     3,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#00-07
    MachineState.ERROR,MachineState.ERROR,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ERROR,#08-0f
    MachineState.ERROR,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START#10-17
)

BIG5_CHAR_LEN_TABLE = (0, 1, 1, 2, 0)

BIG5_SM_MODEL = {'class_table': BIG5_CLS,
                 'class_factor': 5,
                 'state_table': BIG5_ST,
                 'char_len_table': BIG5_CHAR_LEN_TABLE,
                 'name': 'Big5'}

# CP949

CP949_CLS  = (
    1,1,1,1,1,1,1,1, 1,1,1,1,1,1,0,0,  # 00 - 0f
    1,1,1,1,1,1,1,1, 1,1,1,0,1,1,1,1,  # 10 - 1f
    1,1,1,1,1,1,1,1, 1,1,1,1,1,1,1,1,  # 20 - 2f
    1,1,1,1,1,1,1,1, 1,1,1,1,1,1,1,1,  # 30 - 3f
    1,4,4,4,4,4,4,4, 4,4,4,4,4,4,4,4,  # 40 - 4f
    4,4,5,5,5,5,5,5, 5,5,5,1,1,1,1,1,  # 50 - 5f
    1,5,5,5,5,5,5,5, 5,5,5,5,5,5,5,5,  # 60 - 6f
    5,5,5,5,5,5,5,5, 5,5,5,1,1,1,1,1,  # 70 - 7f
    0,6,6,6,6,6,6,6, 6,6,6,6,6,6,6,6,  # 80 - 8f
    6,6,6,6,6,6,6,6, 6,6,6,6,6,6,6,6,  # 90 - 9f
    6,7,7,7,7,7,7,7, 7,7,7,7,7,8,8,8,  # a0 - af
    7,7,7,7,7,7,7,7, 7,7,7,7,7,7,7,7,  # b0 - bf
    7,7,7,7,7,7,9,2, 2,3,2,2,2,2,2,2,  # c0 - cf
    2,2,2,2,2,2,2,2, 2,2,2,2,2,2,2,2,  # d0 - df
    2,2,2,2,2,2,2,2, 2,2,2,2,2,2,2,2,  # e0 - ef
    2,2,2,2,2,2,2,2, 2,2,2,2,2,2,2,0,  # f0 - ff
)

CP949_ST = (
#cls=    0      1      2      3      4      5      6      7      8      9  # previous state =
    MachineState.ERROR,MachineState.START,     3,MachineState.ERROR,MachineState.START,MachineState.START,     4,     5,MachineState.ERROR,     6, # MachineState.START
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR, # MachineState.ERROR
    MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME, # MachineState.ITS_ME
    MachineState.ERROR,MachineState.ERROR,MachineState.START,MachineState.START,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.START,MachineState.START,MachineState.START, # 3
    MachineState.ERROR,MachineState.ERROR,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START, # 4
    MachineState.ERROR,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START, # 5
    MachineState.ERROR,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.ERROR,MachineState.ERROR,MachineState.START,MachineState.START,MachineState.START, # 6
)

CP949_CHAR_LEN_TABLE = (0, 1, 2, 0, 1, 1, 2, 2, 0, 2)

CP949_SM_MODEL = {'class_table': CP949_CLS,
                  'class_factor': 10,
                  'state_table': CP949_ST,
                  'char_len_table': CP949_CHAR_LEN_TABLE,
                  'name': 'CP949'}

# EUC-JP

EUCJP_CLS = (
    4,4,4,4,4,4,4,4,  # 00 - 07
    4,4,4,4,4,4,5,5,  # 08 - 0f
    4,4,4,4,4,4,4,4,  # 10 - 17
    4,4,4,5,4,4,4,4,  # 18 - 1f
    4,4,4,4,4,4,4,4,  # 20 - 27
    4,4,4,4,4,4,4,4,  # 28 - 2f
    4,4,4,4,4,4,4,4,  # 30 - 37
    4,4,4,4,4,4,4,4,  # 38 - 3f
    4,4,4,4,4,4,4,4,  # 40 - 47
    4,4,4,4,4,4,4,4,  # 48 - 4f
    4,4,4,4,4,4,4,4,  # 50 - 57
    4,4,4,4,4,4,4,4,  # 58 - 5f
    4,4,4,4,4,4,4,4,  # 60 - 67
    4,4,4,4,4,4,4,4,  # 68 - 6f
    4,4,4,4,4,4,4,4,  # 70 - 77
    4,4,4,4,4,4,4,4,  # 78 - 7f
    5,5,5,5,5,5,5,5,  # 80 - 87
    5,5,5,5,5,5,1,3,  # 88 - 8f
    5,5,5,5,5,5,5,5,  # 90 - 97
    5,5,5,5,5,5,5,5,  # 98 - 9f
    5,2,2,2,2,2,2,2,  # a0 - a7
    2,2,2,2,2,2,2,2,  # a8 - af
    2,2,2,2,2,2,2,2,  # b0 - b7
    2,2,2,2,2,2,2,2,  # b8 - bf
    2,2,2,2,2,2,2,2,  # c0 - c7
    2,2,2,2,2,2,2,2,  # c8 - cf
    2,2,2,2,2,2,2,2,  # d0 - d7
    2,2,2,2,2,2,2,2,  # d8 - df
    0,0,0,0,0,0,0,0,  # e0 - e7
    0,0,0,0,0,0,0,0,  # e8 - ef
    0,0,0,0,0,0,0,0,  # f0 - f7
    0,0,0,0,0,0,0,5  # f8 - ff
)

EUCJP_ST = (
          3,     4,     3,     5,MachineState.START,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#00-07
     MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,#08-0f
     MachineState.ITS_ME,MachineState.ITS_ME,MachineState.START,MachineState.ERROR,MachineState.START,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#10-17
     MachineState.ERROR,MachineState.ERROR,MachineState.START,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,     3,MachineState.ERROR,#18-1f
          3,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.START,MachineState.START,MachineState.START,MachineState.START#20-27
)

EUCJP_CHAR_LEN_TABLE = (2, 2, 2, 3, 1, 0)

EUCJP_SM_MODEL = {'class_table': EUCJP_CLS,
                  'class_factor': 6,
                  'state_table': EUCJP_ST,
                  'char_len_table': EUCJP_CHAR_LEN_TABLE,
                  'name': 'EUC-JP'}

# EUC-KR

EUCKR_CLS  = (
    1,1,1,1,1,1,1,1,  # 00 - 07
    1,1,1,1,1,1,0,0,  # 08 - 0f
    1,1,1,1,1,1,1,1,  # 10 - 17
    1,1,1,0,1,1,1,1,  # 18 - 1f
    1,1,1,1,1,1,1,1,  # 20 - 27
    1,1,1,1,1,1,1,1,  # 28 - 2f
    1,1,1,1,1,1,1,1,  # 30 - 37
    1,1,1,1,1,1,1,1,  # 38 - 3f
    1,1,1,1,1,1,1,1,  # 40 - 47
    1,1,1,1,1,1,1,1,  # 48 - 4f
    1,1,1,1,1,1,1,1,  # 50 - 57
    1,1,1,1,1,1,1,1,  # 58 - 5f
    1,1,1,1,1,1,1,1,  # 60 - 67
    1,1,1,1,1,1,1,1,  # 68 - 6f
    1,1,1,1,1,1,1,1,  # 70 - 77
    1,1,1,1,1,1,1,1,  # 78 - 7f
    0,0,0,0,0,0,0,0,  # 80 - 87
    0,0,0,0,0,0,0,0,  # 88 - 8f
    0,0,0,0,0,0,0,0,  # 90 - 97
    0,0,0,0,0,0,0,0,  # 98 - 9f
    0,2,2,2,2,2,2,2,  # a0 - a7
    2,2,2,2,2,3,3,3,  # a8 - af
    2,2,2,2,2,2,2,2,  # b0 - b7
    2,2,2,2,2,2,2,2,  # b8 - bf
    2,2,2,2,2,2,2,2,  # c0 - c7
    2,3,2,2,2,2,2,2,  # c8 - cf
    2,2,2,2,2,2,2,2,  # d0 - d7
    2,2,2,2,2,2,2,2,  # d8 - df
    2,2,2,2,2,2,2,2,  # e0 - e7
    2,2,2,2,2,2,2,2,  # e8 - ef
    2,2,2,2,2,2,2,2,  # f0 - f7
    2,2,2,2,2,2,2,0   # f8 - ff
)

EUCKR_ST = (
    MachineState.ERROR,MachineState.START,     3,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#00-07
    MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ERROR,MachineState.ERROR,MachineState.START,MachineState.START #08-0f
)

EUCKR_CHAR_LEN_TABLE = (0, 1, 2, 0)

EUCKR_SM_MODEL = {'class_table': EUCKR_CLS,
                'class_factor': 4,
                'state_table': EUCKR_ST,
                'char_len_table': EUCKR_CHAR_LEN_TABLE,
                'name': 'EUC-KR'}

# EUC-TW

EUCTW_CLS = (
    2,2,2,2,2,2,2,2,  # 00 - 07
    2,2,2,2,2,2,0,0,  # 08 - 0f
    2,2,2,2,2,2,2,2,  # 10 - 17
    2,2,2,0,2,2,2,2,  # 18 - 1f
    2,2,2,2,2,2,2,2,  # 20 - 27
    2,2,2,2,2,2,2,2,  # 28 - 2f
    2,2,2,2,2,2,2,2,  # 30 - 37
    2,2,2,2,2,2,2,2,  # 38 - 3f
    2,2,2,2,2,2,2,2,  # 40 - 47
    2,2,2,2,2,2,2,2,  # 48 - 4f
    2,2,2,2,2,2,2,2,  # 50 - 57
    2,2,2,2,2,2,2,2,  # 58 - 5f
    2,2,2,2,2,2,2,2,  # 60 - 67
    2,2,2,2,2,2,2,2,  # 68 - 6f
    2,2,2,2,2,2,2,2,  # 70 - 77
    2,2,2,2,2,2,2,2,  # 78 - 7f
    0,0,0,0,0,0,0,0,  # 80 - 87
    0,0,0,0,0,0,6,0,  # 88 - 8f
    0,0,0,0,0,0,0,0,  # 90 - 97
    0,0,0,0,0,0,0,0,  # 98 - 9f
    0,3,4,4,4,4,4,4,  # a0 - a7
    5,5,1,1,1,1,1,1,  # a8 - af
    1,1,1,1,1,1,1,1,  # b0 - b7
    1,1,1,1,1,1,1,1,  # b8 - bf
    1,1,3,1,3,3,3,3,  # c0 - c7
    3,3,3,3,3,3,3,3,  # c8 - cf
    3,3,3,3,3,3,3,3,  # d0 - d7
    3,3,3,3,3,3,3,3,  # d8 - df
    3,3,3,3,3,3,3,3,  # e0 - e7
    3,3,3,3,3,3,3,3,  # e8 - ef
    3,3,3,3,3,3,3,3,  # f0 - f7
    3,3,3,3,3,3,3,0   # f8 - ff
)

EUCTW_ST = (
    MachineState.ERROR,MachineState.ERROR,MachineState.START,     3,     3,     3,     4,MachineState.ERROR,#00-07
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ITS_ME,MachineState.ITS_ME,#08-0f
    MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ERROR,MachineState.START,MachineState.ERROR,#10-17
    MachineState.START,MachineState.START,MachineState.START,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#18-1f
         5,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.START,MachineState.ERROR,MachineState.START,MachineState.START,#20-27
    MachineState.START,MachineState.ERROR,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START #28-2f
)

EUCTW_CHAR_LEN_TABLE = (0, 0, 1, 2, 2, 2, 3)

EUCTW_SM_MODEL = {'class_table': EUCTW_CLS,
                'class_factor': 7,
                'state_table': EUCTW_ST,
                'char_len_table': EUCTW_CHAR_LEN_TABLE,
                'name': 'x-euc-tw'}

# GB2312

GB2312_CLS = (
    1,1,1,1,1,1,1,1,  # 00 - 07
    1,1,1,1,1,1,0,0,  # 08 - 0f
    1,1,1,1,1,1,1,1,  # 10 - 17
    1,1,1,0,1,1,1,1,  # 18 - 1f
    1,1,1,1,1,1,1,1,  # 20 - 27
    1,1,1,1,1,1,1,1,  # 28 - 2f
    3,3,3,3,3,3,3,3,  # 30 - 37
    3,3,1,1,1,1,1,1,  # 38 - 3f
    2,2,2,2,2,2,2,2,  # 40 - 47
    2,2,2,2,2,2,2,2,  # 48 - 4f
    2,2,2,2,2,2,2,2,  # 50 - 57
    2,2,2,2,2,2,2,2,  # 58 - 5f
    2,2,2,2,2,2,2,2,  # 60 - 67
    2,2,2,2,2,2,2,2,  # 68 - 6f
    2,2,2,2,2,2,2,2,  # 70 - 77
    2,2,2,2,2,2,2,4,  # 78 - 7f
    5,6,6,6,6,6,6,6,  # 80 - 87
    6,6,6,6,6,6,6,6,  # 88 - 8f
    6,6,6,6,6,6,6,6,  # 90 - 97
    6,6,6,6,6,6,6,6,  # 98 - 9f
    6,6,6,6,6,6,6,6,  # a0 - a7
    6,6,6,6,6,6,6,6,  # a8 - af
    6,6,6,6,6,6,6,6,  # b0 - b7
    6,6,6,6,6,6,6,6,  # b8 - bf
    6,6,6,6,6,6,6,6,  # c0 - c7
    6,6,6,6,6,6,6,6,  # c8 - cf
    6,6,6,6,6,6,6,6,  # d0 - d7
    6,6,6,6,6,6,6,6,  # d8 - df
    6,6,6,6,6,6,6,6,  # e0 - e7
    6,6,6,6,6,6,6,6,  # e8 - ef
    6,6,6,6,6,6,6,6,  # f0 - f7
    6,6,6,6,6,6,6,0   # f8 - ff
)

GB2312_ST = (
    MachineState.ERROR,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START,     3,MachineState.ERROR,#00-07
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ITS_ME,MachineState.ITS_ME,#08-0f
    MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ERROR,MachineState.ERROR,MachineState.START,#10-17
         4,MachineState.ERROR,MachineState.START,MachineState.START,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#18-1f
    MachineState.ERROR,MachineState.ERROR,     5,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ITS_ME,MachineState.ERROR,#20-27
    MachineState.ERROR,MachineState.ERROR,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.START #28-2f
)

# To be accurate, the length of class 6 can be either 2 or 4.
# But it is not necessary to discriminate between the two since
# it is used for frequency analysis only, and we are validating
# each code range there as well. So it is safe to set it to be
# 2 here.
GB2312_CHAR_LEN_TABLE = (0, 1, 1, 1, 1, 1, 2)

GB2312_SM_MODEL = {'class_table': GB2312_CLS,
                   'class_factor': 7,
                   'state_table': GB2312_ST,
                   'char_len_table': GB2312_CHAR_LEN_TABLE,
                   'name': 'GB2312'}

# Shift_JIS

SJIS_CLS = (
    1,1,1,1,1,1,1,1,  # 00 - 07
    1,1,1,1,1,1,0,0,  # 08 - 0f
    1,1,1,1,1,1,1,1,  # 10 - 17
    1,1,1,0,1,1,1,1,  # 18 - 1f
    1,1,1,1,1,1,1,1,  # 20 - 27
    1,1,1,1,1,1,1,1,  # 28 - 2f
    1,1,1,1,1,1,1,1,  # 30 - 37
    1,1,1,1,1,1,1,1,  # 38 - 3f
    2,2,2,2,2,2,2,2,  # 40 - 47
    2,2,2,2,2,2,2,2,  # 48 - 4f
    2,2,2,2,2,2,2,2,  # 50 - 57
    2,2,2,2,2,2,2,2,  # 58 - 5f
    2,2,2,2,2,2,2,2,  # 60 - 67
    2,2,2,2,2,2,2,2,  # 68 - 6f
    2,2,2,2,2,2,2,2,  # 70 - 77
    2,2,2,2,2,2,2,1,  # 78 - 7f
    3,3,3,3,3,2,2,3,  # 80 - 87
    3,3,3,3,3,3,3,3,  # 88 - 8f
    3,3,3,3,3,3,3,3,  # 90 - 97
    3,3,3,3,3,3,3,3,  # 98 - 9f
    #0xa0 is illegal in sjis encoding, but some pages does
    #contain such byte. We need to be more error forgiven.
    2,2,2,2,2,2,2,2,  # a0 - a7
    2,2,2,2,2,2,2,2,  # a8 - af
    2,2,2,2,2,2,2,2,  # b0 - b7
    2,2,2,2,2,2,2,2,  # b8 - bf
    2,2,2,2,2,2,2,2,  # c0 - c7
    2,2,2,2,2,2,2,2,  # c8 - cf
    2,2,2,2,2,2,2,2,  # d0 - d7
    2,2,2,2,2,2,2,2,  # d8 - df
    3,3,3,3,3,3,3,3,  # e0 - e7
    3,3,3,3,3,4,4,4,  # e8 - ef
    3,3,3,3,3,3,3,3,  # f0 - f7
    3,3,3,3,3,0,0,0)  # f8 - ff


SJIS_ST = (
    MachineState.ERROR,MachineState.START,MachineState.START,     3,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#00-07
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,#08-0f
    MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ERROR,MachineState.ERROR,MachineState.START,MachineState.START,MachineState.START,MachineState.START #10-17
)

SJIS_CHAR_LEN_TABLE = (0, 1, 1, 2, 0, 0)

SJIS_SM_MODEL = {'class_table': SJIS_CLS,
               'class_factor': 6,
               'state_table': SJIS_ST,
               'char_len_table': SJIS_CHAR_LEN_TABLE,
               'name': 'Shift_JIS'}

# UCS2-BE

UCS2BE_CLS = (
    0,0,0,0,0,0,0,0,  # 00 - 07
    0,0,1,0,0,2,0,0,  # 08 - 0f
    0,0,0,0,0,0,0,0,  # 10 - 17
    0,0,0,3,0,0,0,0,  # 18 - 1f
    0,0,0,0,0,0,0,0,  # 20 - 27
    0,3,3,3,3,3,0,0,  # 28 - 2f
    0,0,0,0,0,0,0,0,  # 30 - 37
    0,0,0,0,0,0,0,0,  # 38 - 3f
    0,0,0,0,0,0,0,0,  # 40 - 47
    0,0,0,0,0,0,0,0,  # 48 - 4f
    0,0,0,0,0,0,0,0,  # 50 - 57
    0,0,0,0,0,0,0,0,  # 58 - 5f
    0,0,0,0,0,0,0,0,  # 60 - 67
    0,0,0,0,0,0,0,0,  # 68 - 6f
    0,0,0,0,0,0,0,0,  # 70 - 77
    0,0,0,0,0,0,0,0,  # 78 - 7f
    0,0,0,0,0,0,0,0,  # 80 - 87
    0,0,0,0,0,0,0,0,  # 88 - 8f
    0,0,0,0,0,0,0,0,  # 90 - 97
    0,0,0,0,0,0,0,0,  # 98 - 9f
    0,0,0,0,0,0,0,0,  # a0 - a7
    0,0,0,0,0,0,0,0,  # a8 - af
    0,0,0,0,0,0,0,0,  # b0 - b7
    0,0,0,0,0,0,0,0,  # b8 - bf
    0,0,0,0,0,0,0,0,  # c0 - c7
    0,0,0,0,0,0,0,0,  # c8 - cf
    0,0,0,0,0,0,0,0,  # d0 - d7
    0,0,0,0,0,0,0,0,  # d8 - df
    0,0,0,0,0,0,0,0,  # e0 - e7
    0,0,0,0,0,0,0,0,  # e8 - ef
    0,0,0,0,0,0,0,0,  # f0 - f7
    0,0,0,0,0,0,4,5   # f8 - ff
)

UCS2BE_ST  = (
          5,     7,     7,MachineState.ERROR,     4,     3,MachineState.ERROR,MachineState.ERROR,#00-07
     MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,#08-0f
     MachineState.ITS_ME,MachineState.ITS_ME,     6,     6,     6,     6,MachineState.ERROR,MachineState.ERROR,#10-17
          6,     6,     6,     6,     6,MachineState.ITS_ME,     6,     6,#18-1f
          6,     6,     6,     6,     5,     7,     7,MachineState.ERROR,#20-27
          5,     8,     6,     6,MachineState.ERROR,     6,     6,     6,#28-2f
          6,     6,     6,     6,MachineState.ERROR,MachineState.ERROR,MachineState.START,MachineState.START #30-37
)

UCS2BE_CHAR_LEN_TABLE = (2, 2, 2, 0, 2, 2)

UCS2BE_SM_MODEL = {'class_table': UCS2BE_CLS,
                   'class_factor': 6,
                   'state_table': UCS2BE_ST,
                   'char_len_table': UCS2BE_CHAR_LEN_TABLE,
                   'name': 'UTF-16BE'}

# UCS2-LE

UCS2LE_CLS = (
    0,0,0,0,0,0,0,0,  # 00 - 07
    0,0,1,0,0,2,0,0,  # 08 - 0f
    0,0,0,0,0,0,0,0,  # 10 - 17
    0,0,0,3,0,0,0,0,  # 18 - 1f
    0,0,0,0,0,0,0,0,  # 20 - 27
    0,3,3,3,3,3,0,0,  # 28 - 2f
    0,0,0,0,0,0,0,0,  # 30 - 37
    0,0,0,0,0,0,0,0,  # 38 - 3f
    0,0,0,0,0,0,0,0,  # 40 - 47
    0,0,0,0,0,0,0,0,  # 48 - 4f
    0,0,0,0,0,0,0,0,  # 50 - 57
    0,0,0,0,0,0,0,0,  # 58 - 5f
    0,0,0,0,0,0,0,0,  # 60 - 67
    0,0,0,0,0,0,0,0,  # 68 - 6f
    0,0,0,0,0,0,0,0,  # 70 - 77
    0,0,0,0,0,0,0,0,  # 78 - 7f
    0,0,0,0,0,0,0,0,  # 80 - 87
    0,0,0,0,0,0,0,0,  # 88 - 8f
    0,0,0,0,0,0,0,0,  # 90 - 97
    0,0,0,0,0,0,0,0,  # 98 - 9f
    0,0,0,0,0,0,0,0,  # a0 - a7
    0,0,0,0,0,0,0,0,  # a8 - af
    0,0,0,0,0,0,0,0,  # b0 - b7
    0,0,0,0,0,0,0,0,  # b8 - bf
    0,0,0,0,0,0,0,0,  # c0 - c7
    0,0,0,0,0,0,0,0,  # c8 - cf
    0,0,0,0,0,0,0,0,  # d0 - d7
    0,0,0,0,0,0,0,0,  # d8 - df
    0,0,0,0,0,0,0,0,  # e0 - e7
    0,0,0,0,0,0,0,0,  # e8 - ef
    0,0,0,0,0,0,0,0,  # f0 - f7
    0,0,0,0,0,0,4,5   # f8 - ff
)

UCS2LE_ST = (
          6,     6,     7,     6,     4,     3,MachineState.ERROR,MachineState.ERROR,#00-07
     MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,#08-0f
     MachineState.ITS_ME,MachineState.ITS_ME,     5,     5,     5,MachineState.ERROR,MachineState.ITS_ME,MachineState.ERROR,#10-17
          5,     5,     5,MachineState.ERROR,     5,MachineState.ERROR,     6,     6,#18-1f
          7,     6,     8,     8,     5,     5,     5,MachineState.ERROR,#20-27
          5,     5,     5,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,     5,     5,#28-2f
          5,     5,     5,MachineState.ERROR,     5,MachineState.ERROR,MachineState.START,MachineState.START #30-37
)

UCS2LE_CHAR_LEN_TABLE = (2, 2, 2, 2, 2, 2)

UCS2LE_SM_MODEL = {'class_table': UCS2LE_CLS,
                 'class_factor': 6,
                 'state_table': UCS2LE_ST,
                 'char_len_table': UCS2LE_CHAR_LEN_TABLE,
                 'name': 'UTF-16LE'}

# UTF-8

UTF8_CLS = (
    1,1,1,1,1,1,1,1,  # 00 - 07  #allow 0x00 as a legal value
    1,1,1,1,1,1,0,0,  # 08 - 0f
    1,1,1,1,1,1,1,1,  # 10 - 17
    1,1,1,0,1,1,1,1,  # 18 - 1f
    1,1,1,1,1,1,1,1,  # 20 - 27
    1,1,1,1,1,1,1,1,  # 28 - 2f
    1,1,1,1,1,1,1,1,  # 30 - 37
    1,1,1,1,1,1,1,1,  # 38 - 3f
    1,1,1,1,1,1,1,1,  # 40 - 47
    1,1,1,1,1,1,1,1,  # 48 - 4f
    1,1,1,1,1,1,1,1,  # 50 - 57
    1,1,1,1,1,1,1,1,  # 58 - 5f
    1,1,1,1,1,1,1,1,  # 60 - 67
    1,1,1,1,1,1,1,1,  # 68 - 6f
    1,1,1,1,1,1,1,1,  # 70 - 77
    1,1,1,1,1,1,1,1,  # 78 - 7f
    2,2,2,2,3,3,3,3,  # 80 - 87
    4,4,4,4,4,4,4,4,  # 88 - 8f
    4,4,4,4,4,4,4,4,  # 90 - 97
    4,4,4,4,4,4,4,4,  # 98 - 9f
    5,5,5,5,5,5,5,5,  # a0 - a7
    5,5,5,5,5,5,5,5,  # a8 - af
    5,5,5,5,5,5,5,5,  # b0 - b7
    5,5,5,5,5,5,5,5,  # b8 - bf
    0,0,6,6,6,6,6,6,  # c0 - c7
    6,6,6,6,6,6,6,6,  # c8 - cf
    6,6,6,6,6,6,6,6,  # d0 - d7
    6,6,6,6,6,6,6,6,  # d8 - df
    7,8,8,8,8,8,8,8,  # e0 - e7
    8,8,8,8,8,9,8,8,  # e8 - ef
    10,11,11,11,11,11,11,11,  # f0 - f7
    12,13,13,13,14,15,0,0    # f8 - ff
)

UTF8_ST = (
    MachineState.ERROR,MachineState.START,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,     12,   10,#00-07
         9,     11,     8,     7,     6,     5,     4,    3,#08-0f
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#10-17
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#18-1f
    MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,#20-27
    MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,MachineState.ITS_ME,#28-2f
    MachineState.ERROR,MachineState.ERROR,     5,     5,     5,     5,MachineState.ERROR,MachineState.ERROR,#30-37
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#38-3f
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,     5,     5,     5,MachineState.ERROR,MachineState.ERROR,#40-47
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#48-4f
    MachineState.ERROR,MachineState.ERROR,     7,     7,     7,     7,MachineState.ERROR,MachineState.ERROR,#50-57
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#58-5f
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,     7,     7,MachineState.ERROR,MachineState.ERROR,#60-67
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#68-6f
    MachineState.ERROR,MachineState.ERROR,     9,     9,     9,     9,MachineState.ERROR,MachineState.ERROR,#70-77
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#78-7f
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,     9,MachineState.ERROR,MachineState.ERROR,#80-87
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#88-8f
    MachineState.ERROR,MachineState.ERROR,    12,    12,    12,    12,MachineState.ERROR,MachineState.ERROR,#90-97
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#98-9f
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,    12,MachineState.ERROR,MachineState.ERROR,#a0-a7
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#a8-af
    MachineState.ERROR,MachineState.ERROR,    12,    12,    12,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#b0-b7
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,#b8-bf
    MachineState.ERROR,MachineState.ERROR,MachineState.START,MachineState.START,MachineState.START,MachineState.START,MachineState.ERROR,MachineState.ERROR,#c0-c7
    MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR,MachineState.ERROR #c8-cf
)

UTF8_CHAR_LEN_TABLE = (0, 1, 0, 0, 0, 0, 2, 3, 3, 3, 4, 4, 5, 5, 6, 6)

UTF8_SM_MODEL = {'class_table': UTF8_CLS,
                 'class_factor': 16,
                 'state_table': UTF8_ST,
                 'char_len_table': UTF8_CHAR_LEN_TABLE,
                 'name': 'UTF-8'}



from production.box import OrderProductCategoryFilterBox
from production.treebuilder import BoxConfiguration

from ._BoxBuilder import BoxBuilder


class OrderProductCategoryFilterBoxBuilder(BoxBuilder):
    def __init__(self, loggers):
        super(OrderProductCategoryFilterBoxBuilder, self).__init__(loggers)

    def build(self, config):
        # type: (BoxConfiguration) -> OrderProductCategoryFilterBox
        return OrderProductCategoryFilterBox(
            config.name,
            config.sons,
            self.get_extra(config, "AllowedCategory", None),
            self.get_extra(config, "ForbiddenCategory", None),
            self.get_extra(config, "FilterMethod", "any"),
            self.get_logger(config)
        )



from messagebus import TokenPriority, TokenCreator

MESSAGE_GROUP = "E"


def _create_token(token_number):
    return TokenCreator.create_token(
        TokenPriority.low, MESSAGE_GROUP, token_number)


class MessageToken(object):
    TK_ELECTRONIC_PAYMENT_ADD_PAYMENT = _create_token("1")
    TK_ELECTRONIC_PAYMENT_CANCEL_PAYMENT = _create_token("2")
    TK_ELECTRONIC_PAYMENT_FINISH_PAYMENT = _create_token("3")
    TK_ELECTRONIC_PAYMENT_CONNECTIVITY_TEST = _create_token("4")
    TK_ELECTRONIC_PAYMENT_SEND_RESPONSE = _create_token("5")
    TK_ELECTRONIC_PAYMENT_CANCEL_PAYMENT_ORDER = _create_token("6")
    TK_ELECTRONIC_PAYMENT_CONFIRM_FINISH = _create_token("7")
    TK_ELECTRONIC_PAYMENT_GET_ELECTRONIC_TYPES = _create_token("8")



from logging import Logger

from interactor.route.peripheral.print_text.dto import PrintTextRequest
from model.config import Config
from model.error_type import PosValidationErrorsType
from model.exception import ValidationError
from model.repository import PosRepositoryModel
from validation_util import validate_api_key


class PrintTextInteractor(object):
    def __init__(self, logger, config, pos_repository):
        # type: (Logger, Config, PosRepositoryModel) -> None

        self.logger = logger
        self.config = config
        self.pos_repository = pos_repository

    def execute(self, request):
        # type: (PrintTextRequest) -> None

        pos_id = validate_api_key(request, self.config.api_key_mapping)
        if not pos_id:
            raise ValidationError(PosValidationErrorsType.PosIdNotFound)

        self.pos_repository.print_text(pos_id, request.content)



from ._ProductionView import ProductionView
from ._PublishView import PublishView
from ._KdsView import KdsView
from ._PrinterView import PrinterView
from ._OrderXml import OrderXml
from ._CommandView import CommandView



import json

from typing import (
    Any,
    Dict,
    Union
)


class CancelPaymentOrderData(object):
    def __init__(
        self,
        data    # type: Dict[str, Any[Union[int, str, Dict]]]
    ):
        # type: (...) -> None

        tender_detail = data.get("tenderDetail", {})

        self.pos_id = data.get("posId", 0)
        self.order_id = data.get("orderId", None)
        self.fiscal_date = data.get("fiscalDate", "")
        self.fiscal_time = data.get("fiscalTime", "")
        self.tender_type = data.get("tenderType", "")
        self.tender_amount = float(data.get("tenderAmount", 0))
        self.nsu_host = tender_detail.get("NsuHost", "")
        self.card_number = tender_detail.get("CardNumber", "")
        self.tender_id = data.get("tenderId", None)
        self.tenders = json.dumps(tender_detail)
        self.operator_id = data.get("operatorId", None)
        self.electronic_type = data.get("electronicType", None)
        self.auth_code = tender_detail.get("AuthCode", None)
        self.media = tender_detail.get("Media", "")

    def __repr__(
        self
    ):
        # type: (...) -> str

        object_cancel = """
            CancelPaymentData(
                pos_id={},
                order_id={}, 
                fiscal_date="{}",
                fiscal_time="{}",
                tender_type={},
                tender_amount={},
                nsu_host="{}",
                card_number="{}",
                tender_id={},
                tenders="{}",
                operator_id={},
                electronic_type={}
                electronic_type="{}"
                media="{}"
            )
        """

        return object_cancel.format(
            self.pos_id,
            self.order_id,
            self.fiscal_date,
            self.fiscal_time,
            self.tender_type,
            self.tender_amount,
            self.nsu_host,
            self.card_number,
            self.tender_id,
            self.tenders,
            self.operator_id,
            self.electronic_type,
            self.auth_code,
            self.media
        )

    def __str__(
        self
    ):
        # type: (...) -> str
        return self.__repr__()



from abc import abstractmethod

from model.event_dispatch import EventDispatch
from model.log import LogEntry
from model.resync import ResyncRequest
from typing import List


class LogRepositoryModel(object):
    @abstractmethod
    def insert_entry(self, log_entry):
        # type: (LogEntry) -> EventDispatch
        raise NotImplementedError()

    @abstractmethod
    def fetch_resync_events(self, resync_request):
        # type: (ResyncRequest) -> List[EventDispatch]
        raise NotImplementedError()



import logging
import os
import shutil


from logging import Logger
import cfgtools

from cfgtools import Group
from systools import sys_parsefilepath

from helper import (
    config_logger,
    import_pydevd,
)
from mbcontextmessagehandler import (
    MbContextMessageBus,
    MbContextMessageHandler,
)
from messagehandlerbuilder import ElectronicPaymentMessageHandlerBuilder
from msgbus import MBEasyContext

from model import SitefClisitefDoesNotExistsException

COMPONENT_NAME = "ElectronicPayment"
REQUIRED_SERVICES = "StoreWideConfig"


def main():
    loader_path = os.environ["LOADERCFG"]

    import_pydevd(loader_path, 9132, False)
    config_logger(loader_path, COMPONENT_NAME, max_files=100, max_size=12582912, add_error_handler=True)

    logger = logging.getLogger(COMPONENT_NAME)
    config = cfgtools.read(loader_path).find_group("ElectronicPayment")  # type: Group

    _get_sitef_config(logger)

    try:
        message_bus = MbContextMessageBus(MBEasyContext(COMPONENT_NAME))
        message_handler_builder = ElectronicPaymentMessageHandlerBuilder(message_bus, config, logger)
        message_handler = MbContextMessageHandler(
            message_bus,
            config.find_value("ServiceName"),
            "Payment",
            REQUIRED_SERVICES,
            message_handler_builder)
        return message_handler.handle_events()
    except Exception as _:
        logger.exception('exception on start')
        raise


def _get_sitef_config(
    logger,     # type: Logger
):
    # type: (...) -> None

    try:
        bundle_dir_path = _get_bundle_dir_path()

        electronic_payment_sitef_dlls_dir = os.path.join(bundle_dir_path, "electronic-payment/dll/sitef/")
        dir_not_exists = not os.path.exists(electronic_payment_sitef_dlls_dir)
        if dir_not_exists:
            logger.info("Electronic payment sitef dlls path does not exists")
            return

        electronic_payment_sitef_config_path = os.path.join(electronic_payment_sitef_dlls_dir, "clisitef.ini")
        file_already_exists = os.path.isfile(electronic_payment_sitef_config_path)
        if file_already_exists:
            logger.info("Electronic payment clisitef.ini already exists")
            return

        sitef_config_file = os.path.join(bundle_dir_path, "sitef/lib/clisitef.ini")
        file_exists = os.path.isfile(sitef_config_file)
        if not file_exists:
            raise SitefClisitefDoesNotExistsException("Sitef clisitef.ini does not exists")

        shutil.copy(sitef_config_file, electronic_payment_sitef_dlls_dir)
    except Exception as _:
        logger.exception("Error getting clisitef.ini")


def _get_bundle_dir_path():
    # type: (...) -> str

    hv_data_dir = sys_parsefilepath("{HVDATADIR}")
    bundle_dir_path = os.path.join(hv_data_dir, "bundles")
    return bundle_dir_path



from messageprocessorutil import UuidMessageProcessor


class OrderModifiedIgnoredTypesProcessor(UuidMessageProcessor):
    def __init__(self):
        # type: () -> None
        super(OrderModifiedIgnoredTypesProcessor, self).__init__()

    def parse_data(self, data):
        return None

    def call_business(self, _):
        return None

    def format_response(self, message_bus, message, event, input_obj, result):
        # no response
        pass

    def format_exception(self, message_bus, message, event, input_obj, exception):
        # no response
        pass

    def format_parse_exception(self, message_bus, message, event, data, exception):
        # no response
        pass
